# ConfigFileLegacy.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Legacy config file handling object
# Old config file was a wrapper around ConfigParser with some hooks for Sutekh
# purposes
# Copyright 2007 Neil Muller <drnlmuller+sutekh@gmail.com>
# Copyright 2010 Simon Cross <hodgestar+sutekh@gmail.com>
# License: GPL - See COPYRIGHT file for details

"""Legacy config file reader for Sutekh"""

from ConfigParser import RawConfigParser, NoOptionError
from sutekh.gui.ConfigFile import ConfigFile


class ConfigFileLegacy(object):
    """Handle the setup and management of the old config file format.

       Ensure that the needed sections exist, and that sensible
       defaults values are assigned.
       """
    # pylint: disable-msg=R0904
    # We need to provide fine-grained access to all the data,
    # so lots of methods

    __sFiltersSection = 'Filters'
    __sPanesSection = 'Open Panes'
    __sPrefsSection = 'GUI Preferences'
    __sPluginsSection = 'Plugin Options'
    __sPaneInfoSection = 'Pane settings'

    def __init__(self, sFileName):
        self.__sFileName = sFileName
        self.__oConfig = RawConfigParser()
        # Use read to parse file if it exists
        self.__oConfig.read(self.__sFileName)

        if not self.__oConfig.has_section(self.__sPrefsSection):
            self.__oConfig.add_section(self.__sPrefsSection)
            # default to saving on exit
            self.set_save_on_exit(True)

        if 'save pane sizes' not in self.__oConfig.options(
                self.__sPrefsSection):
            self.set_save_precise_pos(True)

        if 'show zero count cards' not in self.__oConfig.options(
                self.__sPrefsSection):
            self.set_show_zero_count_cards(False)

        if 'save window size' not in self.__oConfig.options(
                self.__sPrefsSection):
            self.set_save_window_size(True)

        if 'postfix name display' not in self.__oConfig.options(
                self.__sPrefsSection):
            self.set_postfix_the_display(False)

        if not self.__oConfig.has_section(self.__sPanesSection):
            self.__oConfig.add_section(self.__sPanesSection)
            # No panes information, so we set 'sensible' defaults
            self.add_frame(1, 'physical_card', 'White Wolf Card List', False,
                    False, -1)
            self.add_frame(2, 'Card Text', 'Card Text', False, False, -1)
            self.add_frame(3, 'Card Set List', 'Card Set List', False, False,
                    -1)
            self.add_frame(4, 'physical_card_set', 'My Collection', False,
                    False, -1)

        if not self.__oConfig.has_section(self.__sFiltersSection):
            self.__oConfig.add_section(self.__sFiltersSection)

        self.__iNum = len(self.__oConfig.items(self.__sFiltersSection))

        if not self.__oConfig.has_section(self.__sPluginsSection):
            self.__oConfig.add_section(self.__sPluginsSection)

        if not self.__oConfig.has_section(self.__sPaneInfoSection):
            self.__oConfig.add_section(self.__sPaneInfoSection)

    def _get_bool_key(self, sSection, sKey):
        """Test if a boolean key is set or not"""
        return self.__oConfig.get(sSection, sKey) == 'yes'

    def _set_bool_key(self, sSection, sKey, bValue):
        """Set a boolean key"""
        if bValue:
            self.__oConfig.set(sSection, sKey, 'yes')
        else:
            self.__oConfig.set(sSection, sKey, 'no')

    # pylint: disable-msg=R0201
    # method for backwards compatibility
    def write(self):
        """Write the config file to disk."""
        raise RuntimeError("Writing the old config file no longer supported")

    # pylint: enable-msg=R0201

    def convert(self, sFilename):
        """Return a new-style Sutekh config object."""
        # pylint: disable-msg=R0914
        # Need to convert a lot of options, so we need many variables
        oConfig = ConfigFile(sFilename)
        oConfig.validate()

        # Global Options

        for fSetter, fGetter in (
            (oConfig.set_database_uri, self.get_database_uri),
            (oConfig.set_icon_path, self.get_icon_path),
            (oConfig.set_postfix_the_display, self.get_postfix_the_display),
            (oConfig.set_save_on_exit, self.get_save_on_exit),
            (oConfig.set_save_precise_pos, self.get_save_precise_pos),
            (oConfig.set_save_window_size, self.get_save_window_size),
            (oConfig.set_window_size, self.get_window_size),
            ):
            oValue = fGetter()
            if oValue is not None:
                fSetter(oValue)

        # Open Frames

        oConfig.clear_open_frames()
        for iPaneNumber, sType, sName, bVertical, bClosed, iPos in \
            self.get_all_panes():
            oConfig.add_frame(iPaneNumber, sType, sName, bVertical, bClosed,
                    iPos, None)

        # Filters

        for sKey, sFilter in self.get_filter_keys():
            oConfig.add_filter(sKey, sFilter)

        # Deck Options
        # Note: deck options from .get_all_pane_info() and
        # .get_show_zero_count_cards(). This would require constructing
        # profiles and associating them with frames automatically which is
        # unlikely to create something the user is happy with. The cost of
        # reconfiguring these options should be fairly low.

        # Global Plugin Options

        dKey2PluginMap = {
            'card image path': 'CardImagePlugin',
            'HTML export mode': 'CardSetExportHTML',
            'show starters': 'StarterInfoPlugin',
        }

        for sKey in self.__oConfig.options(self.__sPluginsSection):
            if sKey not in dKey2PluginMap:
                # skip keys that aren't known are are presumablly garbage
                continue
            sPlugin = dKey2PluginMap[sKey]
            sValue = self.get_plugin_key(sKey)
            oConfig.set_plugin_key(sPlugin, sKey, sValue, bCreateSection=True)

        return oConfig

    def pre_save_clear(self):
        """Clear out old saved pos, before saving new stuff"""
        aKeys = self.__oConfig.options(self.__sPanesSection)
        for sKey in aKeys:
            self.__oConfig.remove_option(self.__sPanesSection, sKey)
        aKeys = self.__oConfig.options(self.__sPaneInfoSection)
        for sKey in aKeys:
            self.__oConfig.remove_option(self.__sPaneInfoSection, sKey)

    def get_all_panes(self):
        """Get the all the panes saved in the config file, and their
           positions."""
        aRes = []
        for sKey, sValue in self.__oConfig.items(self.__sPanesSection):
            if not sKey.startswith('pane'):
                # invalid key
                continue
            try:
                iPaneNumber = int(sKey.split(' ')[1])
            except ValueError:
                # invalid key
                continue
            # Type is before 1st colon in the name
            if sValue.find(':') == -1:
                # invalid format, so skip
                continue
            sData, sName = sValue.split(':', 1)
            aData = sData.split('.')
            sType = aData[0]
            sPos = '-1'
            bVertical = False
            bClosed = False
            if len(aData) > 1:
                if aData[1] == 'V':
                    bVertical = True
                    if len(aData) > 2:
                        sPos = aData[2]
                elif aData[1] == 'C':
                    bClosed = True
                    # We aim for minimal impact on the layout of other panes
                    sPos = '0'
                else:
                    sPos = aData[1]
            try:
                iPos = int(sPos)
            except ValueError:
                iPos = -1
            aRes.append((iPaneNumber, sType, sName, bVertical, bClosed, iPos))
        aRes.sort()  # Numbers denote ordering
        return aRes

    def get_all_pane_info(self):
        """Get the pane info (card mode, etc) saved in the config file"""
        dRes = {}
        for tPaneInfo in self.__oConfig.items(self.__sPaneInfoSection):
            # Format is 'pane number' =
            #        Extra Level Mode:Parent Count Mode:Card Mode:Name
            # We can' use the name as a key, since the key isn't case sensitive
            try:
                sExtraLevelMode, sParentCount, sShowCardMode, sName = \
                        tPaneInfo[1].split(':', 3)
                dRes[sName] = (int(sExtraLevelMode), int(sParentCount),
                        int(sShowCardMode))
            except ValueError:
                # skip this one then
                continue
        return dRes

    def get_postfix_the_display(self):
        """Query the 'postfix names' option."""
        return self._get_bool_key(self.__sPrefsSection, 'postfix name display')

    def set_postfix_the_display(self, bPostfix):
        """Set the 'postfix names' option."""
        self._set_bool_key(self.__sPrefsSection, 'postfix name display',
                bPostfix)

    def get_save_on_exit(self):
        """Query the 'save on exit' option."""
        return self._get_bool_key(self.__sPrefsSection, 'save on exit')

    def set_save_on_exit(self, bSaveOnExit):
        """Set the 'save on exit' option."""
        self._set_bool_key(self.__sPrefsSection, 'save on exit', bSaveOnExit)

    def get_save_precise_pos(self):
        """Query the 'save pane sizes' option."""
        return self._get_bool_key(self.__sPrefsSection, 'save pane sizes')

    def set_save_precise_pos(self, bSavePos):
        """Set the 'save pane sizes' option."""
        self._set_bool_key(self.__sPrefsSection, 'save pane sizes', bSavePos)

    def get_save_window_size(self):
        """Query the 'save window size' option."""
        return self._get_bool_key(self.__sPrefsSection, 'save window size')

    def set_save_window_size(self, bSavePos):
        """Set the 'save window size' option."""
        self._set_bool_key(self.__sPrefsSection, 'save window size', bSavePos)

    def get_show_zero_count_cards(self):
        """Query the 'show zero count cards' option."""
        return self._get_bool_key(self.__sPrefsSection,
                'show zero count cards')

    def set_show_zero_count_cards(self, bShowZero):
        """Set the 'show zero count cards' option."""
        self._set_bool_key(self.__sPrefsSection, 'show zero count cards',
                bShowZero)

    # pylint: disable-msg=R0913
    # We need all the info in the arguments here
    def add_frame(self, iFrameNumber, sType, sName, bVertical, bClosed, iPos):
        """Add a frame with the given position info to the config file"""
        sKey = 'pane %d' % iFrameNumber
        sData = sType
        if bVertical:
            sData += '.V'
        elif bClosed:
            sData += '.C'
        if iPos > 0 and self.get_save_precise_pos() and not bClosed:
            sData += '.%d' % iPos
        sValue = '%s:%s' % (sData, sName)
        self.__oConfig.set(self.__sPanesSection, sKey, sValue)

    # pylint: enable-msg=R0913

    def add_pane_info(self, iFrameNumber, sName, tInfo):
        """Save the pane info"""
        sKey = 'pane %d' % iFrameNumber
        iExtraLevelMode, iParentCount, iShowCardMode = tInfo
        sValue = '%d:%d:%d:%s' % (iExtraLevelMode, iParentCount, iShowCardMode,
                sName)
        self.__oConfig.set(self.__sPaneInfoSection, sKey, sValue)

    def set_database_uri(self, sDatabaseURI):
        """Set the configured database URI"""
        self.__oConfig.set(self.__sPrefsSection, "database url", sDatabaseURI)

    def get_database_uri(self):
        """Get database URI from the config file"""
        try:
            sResult = self.__oConfig.get(self.__sPrefsSection, "database url")
        except NoOptionError:
            sResult = None
        return sResult

    def get_icon_path(self):
        """Get the icon path from the config file"""
        try:
            sResult = self.__oConfig.get(self.__sPrefsSection, "icon path")
        except NoOptionError:
            sResult = None
        return sResult

    def set_icon_path(self, sPath):
        """Set the configured icon path"""
        self.__oConfig.set(self.__sPrefsSection, "icon path", sPath)

    def get_window_size(self):
        """Get the saved window size from the config file."""
        try:
            sResult = self.__oConfig.get(self.__sPrefsSection, 'window size')
            iWidth, iHeight = [int(x) for x in sResult.split(',')]
        except NoOptionError:
            iWidth, iHeight = -1, -1
        return iWidth, iHeight

    def save_window_size(self, tSize):
        """Save the current window size."""
        sPos = '%d, %d' % (tSize[0], tSize[1])
        self.__oConfig.set(self.__sPrefsSection, 'window size', sPos)

    # filters section

    def get_filters(self):
        """Get all the filters in the config file."""
        return [x[1] for x in self.__oConfig.items(self.__sFiltersSection)]

    def get_filter_keys(self):
        """Get the keys used to reference the filters."""
        return self.__oConfig.items(self.__sFiltersSection)

    def get_filter(self, sKey):
        """Return the filter associated with the given key or None."""
        if sKey.lower() in self.__oConfig.options(self.__sFiltersSection):
            return self.__oConfig.get(self.__sFiltersSection, sKey)
        else:
            return None

    def add_filter(self, sKey, sFilter):
        """Add a filter to the config file."""
        if sKey.lower() not in self.__oConfig.options(self.__sFiltersSection):
            self.__oConfig.set(self.__sFiltersSection, sKey, sFilter)

    def remove_filter(self, sKey, sFilter):
        """Remove a filter from the file"""
        # Make sure Filer and key match
        if sKey.lower() in self.__oConfig.options(self.__sFiltersSection) and \
                sFilter == self.__oConfig.get(self.__sFiltersSection, sKey):
            self.__oConfig.remove_option(self.__sFiltersSection, sKey)

    def replace_filter(self, sKey, sOldFilter, sNewFilter):
        """Replace a filter in the file with new filter"""
        if sKey.lower() in self.__oConfig.options(self.__sFiltersSection) and \
                sOldFilter == self.__oConfig.get(self.__sFiltersSection, sKey):
            self.__oConfig.remove_option(self.__sFiltersSection, sKey)
            self.__oConfig.set(self.__sFiltersSection, sKey, sNewFilter)

    # plugins section

    def get_plugin_key(self, sKey):
        """Get an option from the plugins section.

           Return None if no option is set
           """
        try:
            sResult = self.__oConfig.get(self.__sPluginsSection, sKey)
        except NoOptionError:
            sResult = None
        return sResult

    def set_plugin_key(self, sKey, sValue):
        """Set a value in the plugin section"""
        self.__oConfig.set(self.__sPluginsSection, sKey, sValue)



# CardListModel.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Simon Cross <hodgestar@gmail.com>
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""The gtk.TreeModel for the card lists."""

import gtk
import gobject
import logging
from sutekh.core.Filters import FilterAndBox, NullFilter, PhysicalCardFilter, \
        make_illegal_filter, CachedFilter
from sutekh.core.Groupings import CardTypeGrouping
from sutekh.core.SutekhObjects import PhysicalCardToAbstractCardAdapter, \
        PhysicalCard, PhysicalCardAdapter, ExpansionNameAdapter, \
        canonical_to_csv
from sutekh.gui.ConfigFile import ConfigFileListener, WW_CARDLIST

EXTRA_LEVEL_OPTION = "extra levels"
EXTRA_LEVEL_LOOKUP = {
    "none": False,
    "expansions": True,
}

USE_ICONS = "show icons for grouping"
HIDE_ILLEGAL = "hide cards not legal for tournament play"


class CardListModelListener(object):
    """Listens to updates, i.e. .load(...), .alter_card_count(...),
       .add_new_card(..) calls, to CardListModels."""
    def load(self, aPhysCards):
        """The CardListModel has reloaded itself. aPhysCards is the list of
           PhysicalCards loaded."""
        pass

    def alter_card_count(self, oCard, iChg):
        """The count of the given card has been altered by iChg.

           oCard: PhysicalCard for the card altered
           """
        pass

    def add_new_card(self, oCard, iCnt):
        """The card has been added to this set iCnt times.

           oCard: PhysicalCard for the cards altered
           """
        pass

    # pylint: disable-msg=R0201
    # Default of returning true is correct
    def check_card_visible(self, _oPhysCard):
        """Check if a card should be shown in the model.

           return False if the card should be hidden, True otherwise.
           oPhysCard is always a physical card."""

        return True


class CardListModel(gtk.TreeStore, ConfigFileListener):
    # pylint: disable-msg=R0904, R0902
    # inherit a lot of public methods for gtk, need local attributes for state
    """Provides a card list specific API for accessing a gtk.TreeStore."""
    # Use spaces to ensure it sorts first
    # Could possibly be more visually distinct, but users can filter
    # on unknown expansions if needed.

    sUnknownExpansion = ExpansionNameAdapter.sUnknownExpansion

    def __init__(self, oConfig):
        # STRING is the card name, INT is the card count
        super(CardListModel, self).__init__(str, int, int, bool, bool,
                gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT,
                gtk.gdk.Color,
                gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)
        # name, count, parent count, showInc, showDec, text_list, icons,
        #       parent_color, AbstractCard, PhysicalCard

        self._cGroupBy = CardTypeGrouping  # grouping class to use
        # base filter defines the card list
        self._oBaseFilter = PhysicalCardFilter()
        self._cCardClass = PhysicalCard  # card class to use
        # Filter to exclude illegal cards. Needs to be defined after
        # we esablish database connections, et al.
        self.oLegalFilter = CachedFilter(make_illegal_filter())
        self._bApplyFilter = False  # whether to apply the select filter
        # additional filters for selecting from the list
        self._oSelectFilter = None
        self._oConfig = oConfig
        self._oConfig.add_listener(self)

        self.dListeners = {}  # dictionary of CardListModelListeners

        self.bExpansions = True
        self.oEmptyIter = None
        self.oIconManager = None
        self.bUseIcons = True
        self._bHideIllegal = True
        self._oController = None

    # pylint: disable-msg=W0212, C0103
    # W0212 - we explicitly allow access via these properties
    # C0103 - we allow these names
    cardclass = property(fget=lambda self: self._cCardClass,
            fset=lambda self, x: setattr(self, '_cCardClass', x))
    groupby = property(fget=lambda self: self._cGroupBy,
            fset=lambda self, x: setattr(self, '_cGroupBy', x))
    basefilter = property(fget=lambda self: self._oBaseFilter,
            fset=lambda self, x: setattr(self, '_oBaseFilter', x))
    hideillegal = property(fget=lambda self: self._bHideIllegal,
            fset=lambda self, x: setattr(self, '_bHideIllegal', x))
    applyfilter = property(fget=lambda self: self._bApplyFilter,
            fset=lambda self, x: setattr(self, '_bApplyFilter', x))
    selectfilter = property(fget=lambda self: self._oSelectFilter,
            fset=lambda self, x: setattr(self, '_oSelectFilter', x))

    frame_id = property(fget=lambda self: WW_CARDLIST,
            doc="Frame ID of the card list (for selecting profiles)")

    # This isn't a card set id, but it's here to support profiles
    cardset_id = property(fget=lambda self: WW_CARDLIST,
            doc="Cardset ID of card list (for selecting profiles)")

    # pylint: enable-msg=W0212, C0103

    def cleanup(self):
        """Remove the config file listener if needed"""
        self._oController = None
        self._oConfig.remove_listener(self)

    def add_listener(self, oListener):
        """Add a listener to the list of interested listeners."""
        self.dListeners[oListener] = None

    def remove_listener(self, oListener):
        """Remove a listener from the list."""
        del self.dListeners[oListener]

    # various utilty functions for checking the model state

    # Helper's for sorting

    def _sort_col(self, _oModel, oIter1, oIter2, iCol):
        """Default sort function for model"""
        oVal1 = self.get_value(oIter1, iCol)
        oVal2 = self.get_value(oIter2, iCol)
        iRes = cmp(oVal1, oVal2)
        if iRes == 0:
            iRes = self.sort_equal_iters(oIter1, oIter2)
        return iRes

    def sort_equal_iters(self, oIter1, oIter2):
        """Default sort on names (card names, expansion names, etc.)"""
        # The default sorting behaviour for equal rows can be changed by
        # hooking into this
        oVal1 = self.get_value(oIter1, 0)
        oVal2 = self.get_value(oIter2, 0)
        # since col 0 is the name column, and we have unique names at all
        # levels (group names, card name, expansion or card set names),
        # this will always give a non-zero result
        return cmp(oVal1, oVal2)

    def enable_sorting(self):
        """Enable default sorting setup"""
        # We don't do this in __init__, as it makes the test suite a
        # lot slower, and the tests don't need sorting anyway.
        # The view calls this to enable sorting. Not elegant, but
        # at least workable.
        self.set_sort_func(0, self._sort_col, 0)
        self.set_sort_func(1, self._sort_col, 1)
        self.set_sort_func(2, self._sort_col, 2)
        # Sort the model on the card name by default
        self.set_sort_column_id(0, gtk.SORT_ASCENDING)

    def set_controller(self, oController):
        """Set the controller"""
        self._oController = oController
        self.update_options(True)

    def get_expansion_info(self, _oCard, dExpanInfo):
        """Get information about expansions"""
        aExpansions = []
        if not self.bExpansions:
            return aExpansions
        for oPhysCard in dExpanInfo:
            aExpansions.append((oPhysCard, ExpansionNameAdapter(oPhysCard)))
        return aExpansions

    def lookup_icons(self, sGroup):
        """Lookup the icons for the group. Method since it's repeated in
           several places in CardSetListModel"""
        if self.oIconManager and self.bUseIcons:
            aTexts, aIcons = self.oIconManager.get_info(sGroup,
                    self.groupby)
        else:
            aTexts = aIcons = []
        return aTexts, aIcons

    # pylint: disable-msg=R0201
    # Method so it can be inherited
    def _fix_group_name(self, sGroup):
        """Fix the None group name"""
        # Utiltiy helper, so we only need to change this once
        if sGroup is None:
            return "<< None >>"
        return sGroup

    # pylint: enable-msg=R0201

    def _set_display_name(self, bPostfix):
        """Set the correct display name for the cards.

           We walk all the 1st level entries, and set the name from the
           card, based on bPostfix."""
        oIter = self.get_iter_first()  # Grouping level items
        while oIter:
            oChildIter = self.iter_children(oIter)
            while oChildIter:
                sName = self.get_card_name_from_iter(oChildIter)
                if bPostfix:
                    sName = canonical_to_csv(sName)
                self.set(oChildIter, 0, sName)
                self.row_changed(self.get_path(oChildIter), oChildIter)
                oChildIter = self.iter_next(oChildIter)
            oIter = self.iter_next(oIter)

    def set_postfix_the_display(self, bPostfix):
        """Respond to config file changes. Passes straight through to
           _set_display_name so we don't need another load."""
        # Disable sorting while we touch everything
        iSortColumn, iSortOrder = self.get_sort_column_id()
        if iSortColumn is not None:
            self.set_sort_column_id(-2, 0)
        self._set_display_name(bPostfix)
        # Enable sorting
        if iSortColumn is not None:
            self.set_sort_column_id(iSortColumn, iSortOrder)

    def load(self):
        # pylint: disable-msg=R0914
        # we use many local variables for clarity
        """Clear and reload the underlying store. For use after initialisation
           or when the filter or grouping changes."""
        self.clear()

        oCardIter = self.get_card_iterator(self.get_current_filter())
        fGetCard, _fGetCount, fGetExpanInfo, oGroupedIter, aCards = \
                self.grouped_card_iter(oCardIter)

        self.oEmptyIter = None

        # Disable sorting while we do the insertions - speeds things up
        iSortColumn, iSortOrder = self.get_sort_column_id()
        if iSortColumn is not None:
            self.set_sort_column_id(-2, 0)

        # Iterate over groups
        bEmpty = True
        bPostfix = self._oConfig.get_postfix_the_display()
        for sGroup, oGroupIter in oGroupedIter:
            # Check for null group
            sGroup = self._fix_group_name(sGroup)

            # Create Group Section
            oSectionIter = self.append(None)

            # Fill in Cards
            for oItem in oGroupIter:
                oCard = fGetCard(oItem)
                oChildIter = self.prepend(oSectionIter)
                # We need to lookup the card directly, since aExpansionInfo
                # may not have the info we need
                # Names will be set by _set_display_name
                sName = oCard.name
                if bPostfix:
                    sName = canonical_to_csv(sName)
                self.set(oChildIter,
                    0, sName,
                    8, oCard,
                    9, PhysicalCardAdapter((oCard, None)),
                )
                aExpansionInfo = self.get_expansion_info(oCard,
                        fGetExpanInfo(oItem))
                for oPhysCard, sExpansion in aExpansionInfo:
                    oExpansionIter = self.append(oChildIter)
                    self.set(oExpansionIter,
                            0, sExpansion,
                            9, oPhysCard,
                            )
                bEmpty = False

            # Update Group Section
            aTexts, aIcons = self.lookup_icons(sGroup)
            if aTexts:
                self.set(oSectionIter, 0, sGroup,
                    5, aTexts, 6, aIcons,
                )
            else:
                self.set(oSectionIter, 0, sGroup)

        if bEmpty:
            # Showing nothing
            self.oEmptyIter = self.append(None)
            sText = self._get_empty_text()
            self.set(self.oEmptyIter, 0, sText)

        # Notify Listeners
        for oListener in self.dListeners:
            oListener.load(aCards)

        # We only re-enable sorting after filling listeners, so sorting on
        # listeners which cache information works properly
        if iSortColumn is not None:
            self.set_sort_column_id(iSortColumn, iSortOrder)

    def get_card_iterator(self, oFilter):
        """Return an interator over the card model.

           The filter is combined with self.basefilter. None may be used to
           retrieve the entire card list (with only the base filter
           restriciting which cards appear).
           """
        oFilter = self.combine_filter_with_base(oFilter)

        return oFilter.select(self.cardclass).distinct()

    def grouped_card_iter(self, oCardIter):
        """Return iterator over the card list grouping.

           Returns a triple of fGetCard (the function used to
           retrieve a card from an item), fGetCount (the function used to
           retrieve a card count from an item) and oGroupedIter (an iterator
           over the card groups)
           """
        aCards = []
        fGetCard = lambda x: x[0]
        fGetCount = lambda x: x[1][0]
        fGetExpanInfo = lambda x: x[1][1]

        # Count by Abstract Card
        dAbsCards = {}

        for oPhysCard in oCardIter:
            # pylint: disable-msg=E1101
            # sqlobject confuses pylint
            if not self.check_card_visible(oPhysCard):
                continue
            oAbsCard = PhysicalCardToAbstractCardAdapter(oPhysCard)
            aCards.append(oPhysCard)
            dAbsCards.setdefault(oAbsCard, [0, {}])
            dAbsCards[oAbsCard][0] += 1
            if self.bExpansions:
                dExpanInfo = dAbsCards[oAbsCard][1]
                dExpanInfo.setdefault(oPhysCard, 0)
                dExpanInfo[oPhysCard] += 1

        aAbsCards = list(dAbsCards.iteritems())

        # Iterate over groups
        return (fGetCard, fGetCount, fGetExpanInfo,
                self.groupby(aAbsCards, fGetCard), aCards)

    def get_current_filter(self):
        """Get the current applied filter.

           This is also responsible for handling the not legal filter case."""
        if self.applyfilter and self._bHideIllegal and self.selectfilter:
            return FilterAndBox([self.selectfilter, self.oLegalFilter])
        elif self._bHideIllegal:
            # either not self.apply, or self.selectfilter is None
            return self.oLegalFilter
        elif self.applyfilter:
            return self.selectfilter
        else:
            return None

    def combine_filter_with_base(self, oOtherFilter):
        """Return the combination of oOtherFilter with the base filter.

           This handles the cases where either filter is None properly."""
        if self.basefilter is None and oOtherFilter is None:
            return NullFilter()
        elif self.basefilter is None:
            return oOtherFilter
        elif oOtherFilter is None:
            return self.basefilter
        else:
            return FilterAndBox([self.basefilter, oOtherFilter])

    def get_card_name_from_path(self, oPath):
        """Get the card name associated with the current path. Handle the
           expansion level transparently."""
        oIter = self.get_iter(oPath)
        return self.get_card_name_from_iter(oIter)

    def get_card_name_from_iter(self, oIter):
        """Get the card name associated with the current iter. Handle the
           expansion level transparently."""
        oCard = self.get_abstract_card_from_iter(oIter)
        if oCard:
            return oCard.name
        # Top level item, so just return the string
        return self.get_name_from_iter(oIter)

    def get_abstract_card_from_path(self, oPath):
        """Get the abstract card name for the current path."""
        oIter = self.get_iter(oPath)
        return self.get_abstract_card_from_iter(oIter)

    def get_physical_card_from_path(self, oPath):
        """Get the physical card name for the current path."""
        try:
            oIter = self.get_iter(oPath)
            return self.get_physical_card_from_iter(oIter)
        except ValueError:
            # Something bad has happened, and we have an invalid path
            # log this and return None
            logging.warn('Invalid path (%s) in get_physical_card_from_path',
                    oPath)
            return None

    def check_card_visible(self, oPhysCard):
        """Returns true if oPhysCard should be shown.

           Used by plugins to allow extra filtering of cards."""
        bResult = True
        for oListener in self.dListeners:
            bResult = bResult and oListener.check_card_visible(oPhysCard)
            if not bResult:
                break  # Failed, so bail on loop
        return bResult

    def get_all_iter_children(self, oIter):
        """Get a list of all the subiters of this iter"""
        aChildIters = []
        oChildIter = self.iter_children(oIter)
        while oChildIter:
            aChildIters.append(oChildIter)
            oChildIter = self.iter_next(oChildIter)
        return aChildIters

    def get_all_from_iter(self, oIter):
        """Get all relevent information about the current iter.

           Returns the tuple (CardName, Expansion info, Card Count,
           depth in the  model), where depth in the model is 1 for the top
           level of cards, and 2 for the expansion level.
           """
        iDepth = self.iter_depth(oIter)
        if iDepth == 0:
            # No card info here
            return None, None, 0, iDepth
        sCardName = self.get_card_name_from_iter(oIter)
        if iDepth == 2:
            sExpansion = self.get_value(oIter, 0)
        else:
            sExpansion = None
        iCount = self.get_value(oIter, 1)
        return sCardName, sExpansion, iCount, iDepth

    def get_all_from_path(self, oPath):
        """Get all relevent information about the current path.

           Conveince wrapper around get_all_from_iter, for use in cases when
           it's easier to get the path than the iter (selections, etc.)
           """
        if oPath:
            # Avoid crashes when oPath is None for some reason
            oIter = self.get_iter(oPath)
            return self.get_all_from_iter(oIter)
        return None, None, None, None

    def get_child_entries_from_path(self, oPath):
        """Return a list of (sExpansion, iCount) pairs for the children of
           this path"""
        oIter = self.get_iter(oPath)
        aChildren = []
        iDepth = self.iter_depth(oIter)
        if iDepth != 1:
            # No children to look at
            return aChildren
        oChildIter = self.iter_children(oIter)
        while oChildIter:
            sExpansion = self.get_value(oChildIter, 0)
            iCount = self.get_value(oChildIter, 1)
            aChildren.append((sExpansion, iCount))
            oChildIter = self.iter_next(oChildIter)
        return aChildren

    def get_inc_dec_flags_from_path(self, oPath):
        """Get the settings of the inc + dec flags for the current path"""
        oIter = self.get_iter(oPath)
        bInc = self.get_value(oIter, 3)
        bDec = self.get_value(oIter, 4)
        return (bInc, bDec)

    def get_exp_name_from_path(self, oPath):
        """Get the expansion information from the model, returing None if this
           is not at a level where the expansion is known."""
        oIter = self.get_iter(oPath)
        if self.iter_depth(oIter) != 2:
            return None
        return self.get_name_from_iter(oIter)

    def get_name_from_iter(self, oIter):
        """Extract the value at oIter from the model, correcting for encoding
           issues."""
        # For some reason the string comes back from the
        # tree store having been encoded *again* despite
        # displaying correctly, so we decode it here.
        # I hope all systems encode with utf-8. :(
        sName = self.get_value(oIter, 0).decode("utf-8")
        return sName

    def get_card_count_from_iter(self, oIter):
        """Return the card count for a given iterator"""
        return self.get_value(oIter, 1)

    def get_parent_count_from_iter(self, oIter):
        """Return the parent count for a given iterator"""
        return self.get_value(oIter, 2)

    def get_abstract_card_from_iter(self, oIter):
        """Extract the Abstract Card at oIter from the model"""
        while self.iter_depth(oIter) > 1:
            # Step up the tree to a level where we store the abstract card
            oIter = self.iter_parent(oIter)
        return self.get_value(oIter, 8)

    def get_physical_card_from_iter(self, oIter):
        """Extract the Physical Card at oIter from the model"""
        return self.get_value(oIter, 9)

    def _get_empty_text(self):
        """Get the correct text for an empty model."""
        if self.get_card_iterator(None).count() == 0:
            sText = 'Empty'
        else:
            sText = 'No Cards found'
        return sText

    def _change_level_mode(self, bLevel):
        """Set which extra information is shown."""
        if self.bExpansions != bLevel:
            self.bExpansions = bLevel
            return True
        return False

    def _change_icon_mode(self, bMode):
        """Set whether icons should be shown."""
        if self.bUseIcons != bMode:
            self.bUseIcons = bMode
            return True
        return False

    def _change_illegal_mode(self, bMode):
        """Set whether illegal cards should be shown."""
        if self._bHideIllegal != bMode:
            self._bHideIllegal = bMode
            return True
        return False

    def update_options(self, bSkipLoad=False):
        """Respond to config file changes"""
        sProfile = self._oConfig.get_profile(WW_CARDLIST, WW_CARDLIST)
        sExpMode = self._oConfig.get_profile_option(WW_CARDLIST, sProfile,
                EXTRA_LEVEL_OPTION).lower()
        bExpMode = EXTRA_LEVEL_LOOKUP.get(sExpMode, True)

        bUseIcons = self._oConfig.get_profile_option(WW_CARDLIST, sProfile,
                USE_ICONS)

        bHideIllegal = self._oConfig.get_profile_option(WW_CARDLIST, sProfile,
                HIDE_ILLEGAL)

        bReloadELM = self._change_level_mode(bExpMode)
        bReloadIcons = self._change_icon_mode(bUseIcons)
        bReloadIllegal = self._change_illegal_mode(bHideIllegal)
        if not bSkipLoad and (bReloadELM or bReloadIcons or bReloadIllegal):
            self._oController.frame.queue_reload()

    # Listen for changes to the cardlist config options

    def profile_option_changed(self, sType, sProfile, sKey):
        """One of the per-deck configuration items changed."""
        if sType != WW_CARDLIST:
            return
        self.update_options()

    def profile_changed(self, sType, sId):
        """A profile option changed with a cardset changed."""
        if sType != WW_CARDLIST or sId != WW_CARDLIST:
            return
        self.update_options()



# CardSetUtilities.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Utility functions for dealing with managing the CardSet Objects"""

from sqlobject import SQLObjectNotFound
from sutekh.core.SutekhObjects import PhysicalCardSet


def get_loop(oCardSet):
    """Return a list names of the card sets in the loop."""
    aLoop = [oCardSet]
    oParent = oCardSet.parent
    while oParent not in aLoop and oParent:
        aLoop.append(oParent)
        oParent = oParent.parent
    if not oParent:
        # No parent case
        return []
    if oParent != oCardSet:
        # oCardSet is not actually part of the loop
        return get_loop(oParent)
    return aLoop


def get_loop_names(oCardSet):
    """Return a list names of the card sets in the loop."""
    aLoopNames = []
    aLoop = get_loop(oCardSet)
    if aLoop:
        aLoopNames = [x.name for x in aLoop]
        aLoopNames.reverse()
    return aLoopNames


def detect_loop(oCardSet):
    """Checks whether the given card set lead to a loop"""
    aSeen = [oCardSet]
    oParent = oCardSet.parent
    while oParent:
        if oParent in aSeen:
            return True  # we have a loop
        aSeen.append(oParent)
        oParent = oParent.parent
    return False  # we've hit none, so no loop


def break_loop(oCardSet):
    """Break the loop that oCardSet leads into"""
    sName = None
    aLoop = get_loop(oCardSet)
    if aLoop:
        # Break the first entry in the loop, as being a good a choice as
        # any
        oCS = aLoop[0]
        oCS.parent = None
        oCS.syncUpdate()
        sName = oCS.name
    return sName


def delete_physical_card_set(sSetName):
    """Unconditionally delete a PCS and its contents"""
    # pylint: disable-msg=E1101
    # SQLObject confuse pylint
    try:
        oCS = PhysicalCardSet.byName(sSetName)
        aChildren = find_children(oCS)
        for oChildCS in aChildren:
            oChildCS.parent = oCS.parent
            oChildCS.syncUpdate()
        for oCard in oCS.cards:
            oCS.removePhysicalCard(oCard)
        PhysicalCardSet.delete(oCS.id)
        return True
    except SQLObjectNotFound:
        return False


def find_children(oCardSet):
    """Find all the children of the given card set"""
    # pylint: disable-msg=E1101
    # SQLObject confuses pylint
    if oCardSet:
        return list(PhysicalCardSet.selectBy(parentID=oCardSet.id))
    return list(PhysicalCardSet.selectBy(parentID=None))


def has_children(oCardSet):
    """Return true if the card set has children"""
    if oCardSet:
        return PhysicalCardSet.selectBy(parentID=oCardSet.id).count() > 0
    return False


def format_cs_list(oParent=None, sIndent=' '):
    """Create a formatted string of all the card sets in the database that
       are children of oParent"""
    aResult = []
    for oCS in sorted(find_children(oParent), key=lambda x: x.name):
        aResult.append(sIndent + oCS.name)
        if has_children(oCS):
            aResult.append(format_cs_list(oCS, sIndent + '   '))
    return '\n'.join(aResult)



# CardSetMenu.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Menu for the CardSet View's
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# Copyright 2006 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Menu for the card set pane"""

import gtk
from sutekh.SutekhUtility import safe_filename
from sutekh.gui.SutekhFileWidget import ExportDialog
from sutekh.io.XmlFileHandling import PhysicalCardSetXmlFile
from sutekh.gui.FilteredViewMenu import CardListMenu
from sutekh.gui.FrameProfileEditor import FrameProfileEditor
from sutekh.gui.LocalProfileEditor import LocalProfileEditor
from sutekh.gui.ConfigFile import ConfigFileListener, CARDSET, FRAME


class CardSetMenu(CardListMenu, ConfigFileListener):
    # pylint: disable-msg=R0904
    # gtk.Widget, so many public methods
    """Card Set Menu.

       Provide the usual menu options, and implement several of the
       card set specific actions - editing card set properties, editng
       annotations, exporting to file, and so on.
       """
    # pylint: disable-msg=R0902
    # R0902 - we are keeping a lot of state, so many instance variables
    def __init__(self, oFrame, oController, oWindow):
        super(CardSetMenu, self).__init__(oFrame, oWindow, oController)
        self.__create_card_set_menu()
        self.create_edit_menu()
        self.create_filter_menu()
        self.create_analyze_menu()
        self.add_plugins_to_menus(self._oFrame)
        self.sort_menu(self._dMenus['Export Card Set'])
        self.sort_menu(self._dMenus['Analyze'])

    # pylint: disable-msg=W0212
    # We allow access via these properties
    name = property(fget=lambda self: self._oController.view.sSetName,
            doc="Associated Card Set Name")

    frame_id = property(fget=lambda self: self._oController.model.frame_id,
            doc="Frame ID of associated card set (for selecting profiles)")

    cardset_id = property(fget=lambda self: self._oController.model.cardset_id,
            doc="Cardset ID of associated card set (for selecting profiles)")

    #pylint: enable-msg=W0212

    # pylint: disable-msg=W0201
    # these methods are called from __init__, so it's OK
    def __create_card_set_menu(self):
        """Create the Actions menu for Card Sets."""
        oMenu = self.create_submenu(self, '_Actions')
        self.create_menu_item("Edit Card Set _Properties", oMenu,
                self._edit_properties)
        self.create_menu_item("_Save Card Set to File", oMenu,
                self._do_export)
        # Submenu for plugins
        self.create_submenu(oMenu, "_Export Card Set")

        oMenu.add(gtk.SeparatorMenuItem())
        self.add_common_actions(oMenu)

        oMenu.add(gtk.SeparatorMenuItem())
        self.create_menu_item("Delete Card Set", oMenu, self._card_set_delete)

    def create_edit_menu(self):
        """Create the 'Edit' menu, and populate it."""
        oMenu = self.create_submenu(self, "_Edit")
        self._oEditable = self.create_check_menu_item('Card Set is Editable',
                oMenu, self.toggle_editable, False, '<Ctrl>e')
        self._oController.view.set_menu(self)
        self.create_menu_item('Copy selection', oMenu, self.copy_selection,
                '<Ctrl>c')
        self._oPaste = self.create_menu_item('Paste', oMenu,
                self._paste_selection, '<Ctrl>v')
        self._oDel = self.create_menu_item('Delete selection', oMenu,
                self._del_selection, 'Delete')
        self._oPaste.set_sensitive(False)
        self._oDel.set_sensitive(False)

        self.create_menu_item('Edit _Local Profile', oMenu,
            self._edit_local_profile)
        self.create_menu_item('Edit _Profiles', oMenu, self._edit_profiles)

        sCardsetProfile = self._oMainWindow.config_file.get_profile(CARDSET,
            self.cardset_id)
        self._oCardsetProfileMenu = self._create_profile_menu(oMenu,
            "Cardset Profile", CARDSET, self._select_cardset_profile,
            sCardsetProfile)

        sFrameProfiles = self._oMainWindow.config_file.get_profile(FRAME,
            self.frame_id)
        self._oFrameProfileMenu = self._create_profile_menu(oMenu,
            "Pane Profile", FRAME, self._select_frame_profile, sFrameProfiles)

        self._oMainWindow.config_file.add_listener(self)

        self.add_edit_menu_actions(oMenu)

    # pylint: enable-msg=W0201

    def cleanup(self):
        """Remove the menu listener"""
        self._oMainWindow.config_file.remove_listener(self)

    def _edit_properties(self, _oWidget):
        """Popup the Edit Properties dialog to change card set properties."""
        # pylint: disable-msg=E1101
        # sqlobject confuses pylint
        self._oController.edit_properties(self)

    def update_card_set_menu(self, oCardSet):
        """Update the menu to reflect changes in the card set name."""
        sNewName = oCardSet.name
        self._oFrame.update_name(sNewName)
        self._oController.view.update_name(sNewName)

    def _do_export(self, _oWidget):
        """Export the card set to the chosen filename."""
        oFileChooser = ExportDialog("Save Card Set As ", self._oMainWindow,
                '%s.xml' % safe_filename(self.name))
        oFileChooser.add_filter_with_pattern('XML Files', ['*.xml'])
        oFileChooser.run()
        sFileName = oFileChooser.get_name()
        if sFileName is not None:
            # User has OK'd us overwriting anything
            oWriter = PhysicalCardSetXmlFile(sFileName)
            oWriter.write(self.name)

    def _card_set_delete(self, _oWidget):
        """Delete the card set."""
        self._oController.delete_card_set()

    def toggle_editable(self, oWidget):
        """Toggle the editable state of the card set."""
        if self._oController.model.bEditable != oWidget.active:
            self._oController.view.toggle_editable(oWidget.active)
            self._oPaste.set_sensitive(oWidget.active)
            self._oDel.set_sensitive(oWidget.active)

    def force_editable_mode(self, bValue):
        """Toggle the editable state of the card set."""
        self._oEditable.set_active(bValue)
        self._oPaste.set_sensitive(bValue)
        self._oDel.set_sensitive(bValue)

    def _del_selection(self, _oWidget):
        """Delete the current selection"""
        self._oController.view.del_selection()

    def _paste_selection(self, _oWidget):
        """Try to paste the current clipboard contents"""
        self._oController.view.do_paste()

    def _edit_local_profile(self, _oWidget):
        """Open an editor for the local cardset profile."""
        oDlg = LocalProfileEditor(self._oMainWindow,
            self._oMainWindow.config_file, self.frame_id, self.cardset_id)
        oDlg.run()

    def _edit_profiles(self, _oWidget):
        """Open an options profiles editing dialog."""
        oDlg = FrameProfileEditor(self._oMainWindow,
            self._oMainWindow.config_file, CARDSET)
        sCurProfile = self._oMainWindow.config_file.get_profile(FRAME,
            self.frame_id)
        if not sCurProfile:
            # We're using the card set profile instead
            sCurProfile = self._oMainWindow.config_file.get_profile(CARDSET,
                    self.cardset_id)
        oDlg.set_selected_profile(sCurProfile)
        oDlg.run()

    def _fix_profile_menu(self):
        """Update the profile menu properly"""
        sCardsetProfile = self._oMainWindow.config_file.get_profile(CARDSET,
            self.cardset_id)
        self._update_profile_group(self._oCardsetProfileMenu, CARDSET,
            self._select_cardset_profile, sCardsetProfile)

        sFrameProfile = self._oMainWindow.config_file.get_profile(FRAME,
            self.frame_id)
        self._update_profile_group(self._oFrameProfileMenu, FRAME,
            self._select_frame_profile, sFrameProfile)

    def _select_cardset_profile(self, oRadio, sProfileKey):
        """Callback to change the profile of the current card set."""
        if oRadio.get_active():
            oConfig = self._oMainWindow.config_file
            oConfig.set_profile(CARDSET, self.cardset_id, sProfileKey)

    def _select_frame_profile(self, oRadio, sProfileKey):
        """Callback to change the profile of the current frame."""
        if oRadio.get_active():
            oConfig = self._oMainWindow.config_file
            oConfig.set_profile(FRAME, self.frame_id, sProfileKey)

    # Respond to profile changes

    def remove_profile(self, sType, _sProfile):
        """A profile has been removed"""
        if sType == CARDSET or sType == FRAME:
            self._fix_profile_menu()

    def profile_option_changed(self, sType, _sProfile, sKey):
        """Update menu if profiles are renamed."""
        if (sType == CARDSET or sType == FRAME) and sKey == 'name':
            self._fix_profile_menu()

    def profile_changed(self, sType, sId):
        if sType == FRAME and sId == self.frame_id:
            self._fix_profile_menu()
        elif sType == CARDSET and sId == self.cardset_id:
            # Relevant when we have multiple copies of the cardset open
            self._fix_profile_menu()



# CardImages.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Adds a frame which will display card images from ARDB in the GUI"""

import gtk
import gobject
import unicodedata
import os
import zipfile
import tempfile
import logging
from sqlobject import SQLObjectNotFound
from sutekh.core.SutekhObjects import IAbstractCard, IExpansion
from sutekh.gui.PluginManager import SutekhPlugin
from sutekh.gui.ProgressDialog import ProgressDialog
from sutekh.gui.CardTextView import CardTextViewListener
from sutekh.gui.BasicFrame import BasicFrame
from sutekh.gui.SutekhDialog import SutekhDialog, do_complaint_buttons, \
        do_complaint_error
from sutekh.gui.AutoScrolledWindow import AutoScrolledWindow
from sutekh.SutekhUtility import prefs_dir, ensure_dir_exists
from sutekh.gui.FileOrUrlWidget import FileOrDirOrUrlWidget
from sutekh.gui.SutekhFileWidget import add_filter


FORWARD, BACKWARD = range(2)
FULL, VIEW_FIXED, FIT = range(3)
RATIO = (225, 300)


def _scale_dims(iImageWidth, iImageHeight, iPaneWidth, iPaneHeight):
    """Rescale the image dimension so they fit in the pane, preserving the
       aspect ratiom."""
    fImageAspectRatio = float(iImageHeight) / float(iImageWidth)
    fPaneAspectRatio = float(iPaneHeight) / float(iPaneWidth)

    if fPaneAspectRatio > fImageAspectRatio:
        # wider
        fDestWidth = iPaneWidth
        fDestHeight = iPaneWidth * fImageAspectRatio
    else:
        fDestHeight = iPaneHeight
        fDestWidth = iPaneHeight / fImageAspectRatio
    return int(fDestWidth), int(fDestHeight)


def _check_file(sFileName):
    """Check if file exists and is readable"""
    bRes = True
    try:
        fTest = file(sFileName, 'rb')
        fTest.close()
    except IOError:
        bRes = False
    return bRes


def _unaccent(sCardName):
    """Remove Unicode accents."""
    # Inspired by a post by renaud turned up by google
    # Unicode Normed decomposed form
    sNormed = unicodedata.normalize('NFD',
            unicode(sCardName.encode('utf8'), encoding='utf-8'))
    # Drop non-ascii characters
    return "".join(b for b in sNormed.encode('utf8') if ord(b) < 128)


class CardImagePopupMenu(gtk.Menu):
    # pylint: disable-msg=R0904
    # R0904 - can't not trigger these warning with pygtk
    """Popup menu for the Card Image Frame"""

    def __init__(self, oFrame, iZoomMode):
        super(CardImagePopupMenu, self).__init__()
        self.oFrame = oFrame
        self.oZoom = gtk.RadioAction('Zoom', 'Show images at original size',
                None, None, FULL)
        self.oZoom.set_group(None)
        self.oViewFixed = gtk.RadioAction('ViewFixed',
                'Show images at fixed size', None, None, VIEW_FIXED)
        self.oViewFixed.set_group(self.oZoom)
        self.oViewFit = gtk.RadioAction('ViewFit', 'Fit images to the pane',
                None, None, FIT)
        self.oViewFit.set_group(self.oZoom)
        self.oNext = gtk.Action('NextExp', 'Show next expansion image', None,
                None)
        self.oPrev = gtk.Action('PrevExp', 'Show previous expansion image',
                None, None)

        self.oPrev.connect('activate', self.cycle_expansion, BACKWARD)
        self.oNext.connect('activate', self.cycle_expansion, FORWARD)
        self.oViewFit.connect('activate', self.set_zoom, FIT)
        self.oZoom.connect('activate', self.set_zoom, FULL)
        self.oViewFixed.connect('activate', self.set_zoom, VIEW_FIXED)

        if iZoomMode == FULL:
            self.oZoom.set_active(True)
        elif iZoomMode == VIEW_FIXED:
            self.oViewFixed.set_active(True)
        elif iZoomMode == FIT:
            self.oViewFit.set_active(True)

        self.add(self.oViewFit.create_menu_item())
        self.add(self.oViewFixed.create_menu_item())
        self.add(self.oZoom.create_menu_item())
        self.add(self.oNext.create_menu_item())
        self.add(self.oPrev.create_menu_item())

    def set_show_expansion_state(self, bValue):
        """Grey out the expansion menus if needed"""
        self.oNext.set_sensitive(bValue)
        self.oPrev.set_sensitive(bValue)

    def cycle_expansion(self, _oWidget, iDir):
        """Change the expansion as requested."""
        assert(iDir in (BACKWARD, FORWARD))
        self.oFrame.do_cycle_expansion(iDir)

    def set_zoom(self, _oWidget, iScale):
        """Change the drawing mode."""
        assert(iScale in (FULL, VIEW_FIXED, FIT))
        self.oFrame.set_zoom_mode(iScale)


class CardImageFrame(BasicFrame, CardTextViewListener):
    # pylint: disable-msg=R0904, R0902
    # R0904 - can't not trigger these warning with pygtk
    # R0902 - we need to keep quite a lot of internal state
    """Frame which displays the image.

       We wrap a gtk.Image in an EventBox (for focus & DnD events)
       and a Viewport (for scrolling)
       """

    sMenuFlag = 'Card Image Frame'

    def __init__(self, oImagePlugin):
        super(CardImageFrame, self).__init__(oImagePlugin.parent)
        self._oImagePlugin = oImagePlugin
        oVBox = gtk.VBox(False, 2)
        oBox = gtk.EventBox()
        self.oExpansionLabel = gtk.Label()
        oVBox.pack_start(self.oExpansionLabel, False, False)
        oVBox.pack_start(oBox)
        self._oView = AutoScrolledWindow(oVBox, True)
        self._oView.get_hadjustment().connect('changed', self._pane_adjust)
        self._oView.get_vadjustment().connect('changed', self._pane_adjust)
        self._oImage = gtk.Image()
        self._oImage.set_from_stock(gtk.STOCK_MISSING_IMAGE,
                gtk.ICON_SIZE_DIALOG)
        oBox.add(self._oImage)

        # Enable DnD handling, same as for BasicFrame
        self.set_drag_handler(oBox)
        self.set_drop_handler(oBox)
        oBox.connect('button-press-event', self.__cycle_expansion)

        self.__sPrefsPath = self._oImagePlugin.get_config_item(
                'card image path')
        if self.__sPrefsPath is None:
            self.__sPrefsPath = os.path.join(prefs_dir('Sutekh'), 'cardimages')
            self._oImagePlugin.set_config_item('card image path',
                    self.__sPrefsPath)
        self.__bShowExpansions = self.__have_expansions()
        self.__sCurExpansion = ''
        self.__aExpansions = []
        self.__iExpansionPos = 0
        self.__sCardName = ''
        self.__iZoomMode = FIT
        self._tPaneSize = (0, 0)

    type = property(fget=lambda self: "Card Image Frame", doc="Frame Type")

    def __have_expansions(self, sTestPath=''):
        """Test if directory contains expansion/image structure used by ARDB"""
        if sTestPath == '':
            sTestFile = os.path.join(self.__sPrefsPath, 'bh', 'acrobatics.jpg')
        else:
            sTestFile = os.path.join(sTestPath, 'bh', 'acrobatics.jpg')
        return _check_file(sTestFile)

    def __convert_expansion(self, sExpansionName):
        """Convert the Full Expansion name into the abbreviation needed."""
        if sExpansionName == '' or not self.__bShowExpansions:
            return ''
        # pylint: disable-msg=E1101
        # pylint doesn't pick up IExpansion methods correctly
        try:
            oExpansion = IExpansion(sExpansionName)
        except SQLObjectNotFound:
            # This can happen because we cache the expansion name and
            # a new database import may cause that to vanish.
            # We return just return a blank path segment, as the safest choice
            logging.warn('Expansion %s no longer found in the database',
                    sExpansionName)
            return ''
        # special case Anarchs and alastors due to promo hack shortname
        if oExpansion.name == 'Anarchs and Alastors Storyline':
            sExpName = oExpansion.name.lower()
        else:
            sExpName = oExpansion.shortname.lower()
        # Normalise for storyline cards
        sExpName = sExpName.replace(' ', '_').replace("'", '')
        return sExpName

    def __set_expansion_info(self, sCardName):
        """Set the expansion info."""
        # pylint: disable-msg=E1101
        # pylint doesn't pick up IAbstractCard methods correctly
        try:
            oAbsCard = IAbstractCard(sCardName)
            bHasInfo = len(oAbsCard.rarity) > 0
        except SQLObjectNotFound:
            bHasInfo = False
        if bHasInfo:
            aExp = [oP.expansion.name for oP in oAbsCard.rarity]
            self.__aExpansions = sorted(list(set(aExp)))  # remove duplicates
            self.__iExpansionPos = 0
            self.__sCurExpansion = self.__aExpansions[0]
        else:
            self.__sCurExpansion = ''
            self.__aExpansions = []
            self.__iExpansionPos = 0

    def __redraw(self, bPause):
        """Redraw the current card"""
        # If further events are pending, don't try and redraw
        if bPause and gtk.gdk.events_pending():
            return
        sFullFilename = self.__convert_cardname()
        self.__load_image(sFullFilename)

    def __convert_cardname(self):
        """Convert sCardName to the form used by the card image list"""
        sCurExpansionPath = self.__convert_expansion(self.__sCurExpansion)
        sFilename = _unaccent(self.__sCardName)
        if sFilename.startswith('the '):
            sFilename = sFilename[4:] + 'the'
        elif sFilename.startswith('an '):
            sFilename = sFilename[3:] + 'an'
        sFilename = sFilename.replace('(advanced)', 'adv')
        # Should probably do this via translate
        for sChar in (" ", ".", ",", "'", "(", ")", "-", ":", "!", '"', "/"):
            sFilename = sFilename.replace(sChar, '')
        sFilename = sFilename + '.jpg'
        return os.path.join(self.__sPrefsPath, sCurExpansionPath, sFilename)

    def __load_image(self, sFullFilename):
        """Load an image into the pane, show broken image if needed"""
        self._oImage.set_alignment(0.5, 0.5)  # Centre image
        try:
            if self.__bShowExpansions:
                self.oExpansionLabel.set_markup('<i>Image from expansion :'
                        ' </i> %s' % self.__sCurExpansion)
                self.oExpansionLabel.show()
                # pylint: disable-msg=E1101
                # pylint doesn't pick up allocation methods correctly
                iHeightOffset = self.oExpansionLabel.allocation.height + 2
            else:
                self.oExpansionLabel.hide()  # config chanes can cause this
                iHeightOffset = 0
            oPixbuf = gtk.gdk.pixbuf_new_from_file(sFullFilename)
            iWidth = oPixbuf.get_width()
            iHeight = oPixbuf.get_height()
            if self.__iZoomMode == FIT:
                # Need to fix aspect ratios
                iPaneHeight = self._oView.get_vadjustment().page_size - \
                        iHeightOffset
                iPaneWidth = self._oView.get_hadjustment().page_size
                # don't centre image under label
                self._oImage.set_alignment(0, 0.5)
                iDestWidth, iDestHeight = _scale_dims(iWidth, iHeight,
                        iPaneWidth, iPaneHeight)
                if iDestWidth > 0 and iDestHeight > 0:
                    self._oImage.set_from_pixbuf(oPixbuf.scale_simple(
                        iDestWidth, iDestHeight, gtk.gdk.INTERP_HYPER))
                    self._tPaneSize = (self._oView.get_hadjustment().page_size,
                            self._oView.get_vadjustment().page_size)
            elif self.__iZoomMode == VIEW_FIXED:
                iDestWidth, iDestHeight = _scale_dims(iWidth, iHeight,
                        RATIO[0], RATIO[1])
                self._oImage.set_from_pixbuf(oPixbuf.scale_simple(iDestWidth,
                    iDestHeight, gtk.gdk.INTERP_HYPER))
            else:
                # Full size, so no scaling
                self._oImage.set_from_pixbuf(oPixbuf)

        except gobject.GError:
            self._oImage.set_from_stock(gtk.STOCK_MISSING_IMAGE,
                    gtk.ICON_SIZE_DIALOG)
        self._oImage.queue_draw()

    def check_images(self, sTestPath=''):
        """Check if dir contains images in the right structure"""
        self.__bShowExpansions = self.__have_expansions(sTestPath)
        if self.__bShowExpansions:
            return True
        if sTestPath == '':
            sTestFile = os.path.join(self.__sPrefsPath, 'acrobatics.jpg')
        else:
            sTestFile = os.path.join(sTestPath, 'acrobatics.jpg')
        return _check_file(sTestFile)

    def update_config_path(self, sNewPath):
        """Update the path we use to search for expansions."""
        self.__sPrefsPath = sNewPath
        self._oImagePlugin.set_config_item('card image path', sNewPath)
        self.__bShowExpansions = self.__have_expansions()

    def set_card_text(self, oPhysCard):
        """Set the image in response to a set card name event."""
        if not oPhysCard:
            return
        sCardName = oPhysCard.abstractCard.canonicalName
        sExpansionName = ''
        if oPhysCard.expansion:
            sExpansionName = oPhysCard.expansion.name
        if sCardName != self.__sCardName:
            self.__set_expansion_info(sCardName)
            self.__sCardName = sCardName
        if len(self.__aExpansions) > 0:
            if sExpansionName in self.__aExpansions:
                # Honour expansion from set_card_text
                self.__sCurExpansion = sExpansionName
                self.__iExpansionPos = self.__aExpansions.index(sExpansionName)
            else:
                # Set self.__sCurExpansion to an existing image, if possible
                self.__iExpansionPos = 0
                bFound = False
                while not bFound and \
                        self.__iExpansionPos < len(self.__aExpansions):
                    self.__sCurExpansion = \
                            self.__aExpansions[self.__iExpansionPos]
                    sFullFilename = self.__convert_cardname()
                    if _check_file(sFullFilename):
                        bFound = True
                    else:
                        self.__iExpansionPos += 1
                if not bFound:
                    self.__sCurExpansion = self.__aExpansions[0]
                    self.__iExpansionPos = 0
        self.__redraw(False)

    def do_cycle_expansion(self, iDir):
        """Change the expansion image to a different one in the list."""
        if len(self.__aExpansions) < 2 or not self.__bShowExpansions:
            return  # nothing to scroll through
        if iDir == FORWARD:
            self.__iExpansionPos += 1
            if self.__iExpansionPos >= len(self.__aExpansions):
                self.__iExpansionPos = 0
        elif iDir == BACKWARD:
            self.__iExpansionPos -= 1
            if self.__iExpansionPos < 0:
                self.__iExpansionPos = len(self.__aExpansions) - 1
        self.__sCurExpansion = self.__aExpansions[self.__iExpansionPos]
        self.__redraw(False)

    def set_zoom_mode(self, iScale):
        """Update the zoom mode."""
        self.__iZoomMode = iScale
        self.__redraw(False)

    def __cycle_expansion(self, _oWidget, oEvent):
        """On a button click, move to the next expansion."""
        if oEvent.type != gtk.gdk.BUTTON_PRESS:
            return True  # don't jump twice on double or triple clicks
        if oEvent.button == 1:
            self.do_cycle_expansion(FORWARD)
        elif oEvent.button == 3:
            # Do context menu
            oPopupMenu = CardImagePopupMenu(self, self.__iZoomMode)
            oPopupMenu.set_show_expansion_state(self.__bShowExpansions and
                    len(self.__aExpansions) > 1)
            oPopupMenu.popup(None, None, None, oEvent.button, oEvent.time)
        return True

    def _pane_adjust(self, _oAdjust):
        """Redraw the image if needed when the pane size changes."""
        if self.__iZoomMode == FIT:
            tCurSize = (self._oView.get_hadjustment().page_size,
                self._oView.get_vadjustment().page_size)
            if tCurSize[0] != self._tPaneSize[0] or \
                    tCurSize[1] != self._tPaneSize[1]:
                self.__redraw(True)

    def get_menu_name(self):
        """Return the menu key"""
        return self.sMenuFlag


class ImageConfigDialog(SutekhDialog):
    # pylint: disable-msg=R0904
    # R0904 - gtk Widget, so has many public methods
    """Dialog for configuring the Image plugin."""

    sDefaultUrl = 'http://csillagmag.hu/upload/pictures.zip'

    def __init__(self, oImagePlugin, bFirstTime=False):
        super(ImageConfigDialog, self).__init__('Configure Card Images Plugin',
                oImagePlugin.parent,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL,
                    gtk.RESPONSE_CANCEL))
        oDescLabel = gtk.Label()
        if not bFirstTime:
            oDescLabel.set_markup('<b>Choose how to configure the cardimages '
                    'plugin</b>')
        else:
            oDescLabel.set_markup('<b>Choose how to configure the cardimages '
                    'plugin</b>\nChoose cancel to skip configuring the '
                    'images plugin\nYou will not be prompted again')
        sDefaultDir = oImagePlugin.get_config_item('card image path')
        self.oChoice = FileOrDirOrUrlWidget(oImagePlugin.parent,
                "Choose location for "
                "images file", "Choose image directory", sDefaultDir,
                {'csillagbolcselet.hu': self.sDefaultUrl})
        add_filter(self.oChoice, 'Zip Files', ['*.zip', '*.ZIP'])
        # pylint: disable-msg=E1101
        # pylint doesn't pick up vbox methods correctly
        self.vbox.pack_start(oDescLabel, False, False)
        self.vbox.pack_start(self.oChoice, False, False)
        self.set_size_request(400, 200)

        self.show_all()

    def get_data(self):
        """Get the results of the users choice."""

        sFile, _bUrl, bDir = self.oChoice.get_file_or_dir_or_url()
        if bDir:
            # Just return the name the user chose
            return sFile, True
        if sFile:
            oOutFile = tempfile.TemporaryFile()
            self.oChoice.get_binary_data(oOutFile)
            return oOutFile, False
        return None, None


class CardImagePlugin(SutekhPlugin):
    """Plugin providing access to CardImageFrame."""
    dTableVersions = {}
    aModelsSupported = ("MainWindow",)

    dGlobalConfig = {
        'card image path': 'string(default=None)',
    }

    _sMenuFlag = CardImageFrame.sMenuFlag

    # pylint: disable-msg=W0142
    # ** magic OK here
    def __init__(self, *args, **kwargs):
        super(CardImagePlugin, self).__init__(*args, **kwargs)
        self.oImageFrame = None
        self._oReplaceItem = None
        self._oAddItem = None
        self._oConfigMenuItem = None

    image_frame = property(fget=lambda self: self.oImageFrame,
            doc="The image frame")

    def init_image_frame(self):
        """Setup the image frame."""
        if not self.oImageFrame:
            self.oImageFrame = CardImageFrame(self)
            self.oImageFrame.set_title(self._sMenuFlag)
            self.oImageFrame.add_parts()

    def get_menu_item(self):
        """Overrides method from base class.

           Adds the menu item on the MainWindow if the images can be found.
           """
        if not self.check_versions() or not self.check_model_type():
            return None
        self.init_image_frame()
        # Add listener
        self.parent.card_text_pane.view.add_listener(self.oImageFrame)
        self._oReplaceItem = gtk.MenuItem("Replace with Card Image Frame")
        self._oReplaceItem.connect("activate", self.replace_pane)

        self._oAddItem = gtk.MenuItem("Add Card Image Frame")
        self._oAddItem.connect("activate", self.add_pane)
        self.parent.add_to_menu_list('Card Image Frame',
                self.add_image_frame_active)
        self._oConfigMenuItem = gtk.MenuItem(
                "Download or Configure Card Images")
        self._oConfigMenuItem.connect("activate", self.config_activate)
        if not self.image_frame.check_images():
            # Don't allow the menu option if we can't find the images
            self.add_image_frame_active(False)
        return [('Data Downloads', self._oConfigMenuItem),
                ('Add Pane', self._oAddItem),
                ('Replace Pane', self._oReplaceItem)]

    def setup(self):
        """Prompt the user to download/setup images the first time"""
        sPrefsPath = self.get_config_item('card image path')
        if not os.path.exists(sPrefsPath):
            # Looks like the first time
            oDialog = ImageConfigDialog(self, True)
            self.handle_response(oDialog)
            # Path may have been changed, so we need to requery config file
            sPrefsPath = self.get_config_item('card image path')
            # Don't get called next time
            ensure_dir_exists(sPrefsPath)

    def config_activate(self, _oMenuWidget):
        """Configure the plugin dialog."""
        oDialog = ImageConfigDialog(self, False)
        self.handle_response(oDialog)

    def handle_response(self, oDialog):
        """Handle the response from the config dialog"""
        iResponse = oDialog.run()
        bActivateMenu = False
        if iResponse == gtk.RESPONSE_OK:
            oFile, bDir = oDialog.get_data()
            if bDir:
                # New directory
                if self._accept_path(oFile):
                    # Update preferences
                    self.image_frame.update_config_path(oFile)
                    bActivateMenu = True
            elif oFile:
                if self._unzip_file(oFile):
                    bActivateMenu = True
                else:
                    do_complaint_error('Unable to successfully unzip data')
                oFile.close()  # clean up temp file
            else:
                # Unable to get data
                do_complaint_error('Unable to configure card images plugin')
        if bActivateMenu:
            # Update the menu display if needed
            if not self.parent.is_open_by_menu_name(self._sMenuFlag):
                # Pane is not open, so try to enable menu
                self.add_image_frame_active(True)
        # get rid of the dialog
        oDialog.destroy()

    def _unzip_file(self, oFile):
        """Unzip a file containing the images."""
        try:
            oZipFile = zipfile.ZipFile(oFile)
        except zipfile.BadZipfile:
            return False
        return self._unzip_heart(oZipFile)

    def _unzip_heart(self, oZipFile):
        """Heavy lifting of unzipping a file"""
        sPrefsPath = self.get_config_item('card image path')
        ensure_dir_exists(sPrefsPath)
        iNumber = len(oZipFile.infolist())
        if iNumber < 300:
            # zipfile too short, so don't bother
            return False
        oProgressDialog = ProgressDialog()
        oProgressDialog.set_description('Unzipping')
        iCur = 0
        for oItem in oZipFile.infolist():
            iCur += 1
            oProgressDialog.update_bar(float(iCur) / iNumber)
            oData = oZipFile.read(oItem.filename)
            # Empty data represents a folder
            # (or at any rate a file to be skipped)
            if not oData:
                continue
            # Should I also test for cardimages\ ?
            sFileName = os.path.join(sPrefsPath,
                    oItem.filename.replace('cardimages/', ''))
            sFileName = sFileName.replace('/', os.path.sep)
            sDir = os.path.dirname(sFileName)
            ensure_dir_exists(sDir)
            oOutputFile = file(sFileName, 'wb')
            oOutputFile.write(oData)
            oOutputFile.close()
        oProgressDialog.destroy()
        if self.image_frame.check_images(sPrefsPath):
            return True
        return False

    def _accept_path(self, sTestPath):
        """Check if the path from user is OK."""
        if sTestPath is not None:
            # Test if path has images
            if not self.image_frame.check_images(sTestPath):
                iQuery = do_complaint_buttons(
                        "Folder does not seem to contain images\n"
                        "Are you sure?", gtk.MESSAGE_QUESTION,
                        (gtk.STOCK_YES, gtk.RESPONSE_YES,
                            gtk.STOCK_NO, gtk.RESPONSE_NO))
                if iQuery == gtk.RESPONSE_NO:
                    # Treat as cancelling
                    return False
            return True
        return False  # No path, can't be OK

    def add_image_frame_active(self, bValue):
        """Toggle the sensitivity of the menu item."""
        if bValue and not self.image_frame.check_images():
            # Can only be set true if check_images returns true
            self._oReplaceItem.set_sensitive(False)
            self._oAddItem.set_sensitive(False)
        else:
            self._oReplaceItem.set_sensitive(bValue)
            self._oAddItem.set_sensitive(bValue)

    def get_frame_from_config(self, sType):
        """Add the frame if it's been saved in the config file."""
        if sType == self._sMenuFlag:
            return self.image_frame
        else:
            return None

    def replace_pane(self, _oWidget):
        """Handle replacing a frame to the main window if required"""
        if not self.parent.is_open_by_menu_name(self._sMenuFlag):
            oNewPane = self.parent.focussed_pane
            if oNewPane:
                self.image_frame.set_unique_id()
                self.parent.replace_frame(oNewPane, self.image_frame)

    def add_pane(self, _oWidget):
        """Handle adding the frame to the main window if required"""
        if not self.parent.is_open_by_menu_name(self._sMenuFlag):
            oNewPane = self.parent.add_pane_end()
            self.image_frame.set_unique_id()
            self.parent.replace_frame(oNewPane, self.image_frame)


plugin = CardImagePlugin



# SutekhFileWidget.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""FileChooserWidget and FileChooserDialog wrapper for Sutekh

   Provides base class for the file widgets and dialogs used in Sutekh -
   ensures set_name is called consistently, and updates the global working
   directory as appropriate
   """

import gtk


def _changed_dir(oFileChooser, oParentWin):
    """Update parent's working dir when the folder changes."""
    if oFileChooser.get_mapped():
        # If we're not mapped, we can't be responsible for changing
        # the directory, so we shouldn't be fiddling with the global
        # value. The mapped call will ensure everything gets set
        # correctly when it's our turn to shine
        oParentWin.set_working_dir(oFileChooser.get_current_folder())


def add_filter(oFileChooser, sFilterName, aFilterPatterns):
    """Add  filter to the widget, using the list of patterns in
       aFilterPatterns"""
    oFilter = gtk.FileFilter()
    oFilter.set_name(sFilterName)
    for sPattern in aFilterPatterns:
        oFilter.add_pattern(sPattern)
    oFileChooser.add_filter(oFilter)
    oFileChooser.set_filter(oFilter)
    return oFilter


def _mapped(oFileWidget, oParent):
    """Update the dialogs working dir when the widget is shown.

       We need this since the working dir will often change between
       button/dialog creation, and the dialog being popped up.
       """
    sWorkingDir = oParent.get_working_dir()
    if sWorkingDir:
        oFileWidget.set_current_folder(sWorkingDir)


class SutekhFileDialog(gtk.FileChooserDialog):
    # pylint: disable-msg=R0904
    # gtk widget, so has many public methods
    """Wrapper for the gtk.FileChooseDialog which updates the
       working dir of Sutekh."""

    def __init__(self, oParent, sTitle, oAction=gtk.FILE_CHOOSER_ACTION_OPEN,
            oButtons=None):
        super(SutekhFileDialog, self).__init__(sTitle, oParent, oAction,
                oButtons)
        self.set_name('Sutekh.dialog')
        # We don't set the working directory here, since that
        # doesn't always work as expected and instead we rely
        # on the mapped signal to fix stuff
        self.connect('current-folder-changed', _changed_dir, oParent)
        self.connect('show', _mapped, oParent)
        self._oAllFilter = add_filter(self, 'All Files', ['*'])

    def add_filter_with_pattern(self, sName, aFilterPatterns):
        """Add a filter named sName to this widget, using the patterns
           in aFilterPatterns."""
        return add_filter(self, sName, aFilterPatterns)

    def default_filter(self):
        """Set the filter to be the default all files filter"""
        self.set_filter(self._oAllFilter)


class SutekhFileWidget(gtk.FileChooserWidget):
    # pylint: disable-msg=R0904
    # gtk widget, so has many public methods
    """Wrapper for the gtk.FileChooseWidget which updates the
       working dir of Sutekh."""
    def __init__(self, oParent, oAction=gtk.FILE_CHOOSER_ACTION_OPEN):
        super(SutekhFileWidget, self).__init__(oAction)
        sWorkingDir = oParent.get_working_dir()
        if sWorkingDir:
            self.set_current_folder(sWorkingDir)
        self.connect('current-folder-changed', _changed_dir, oParent)
        self.connect('show', _mapped, oParent)
        self._oAllFilter = add_filter(self, 'All Files', ['*'])

    def add_filter_with_pattern(self, sName, aFilterPatterns):
        """Add a filter named sName to this widget, using the patterns
           in aFilterPatterns."""
        add_filter(self, sName, aFilterPatterns)

    def default_filter(self):
        """Set the filter to be the default all files filter"""
        self.set_filter(self._oAllFilter)


class SutekhFileButton(gtk.FileChooserButton):
    # pylint: disable-msg=R0904
    # gtk widget, so has many public methods
    """Wrapper class for gtk.FileChooserButton which updates the
       working directory."""

    def __init__(self, oParent, sTitle):
        self.oDialog = SutekhFileDialog(oParent, sTitle,
                oButtons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
                    gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
        super(SutekhFileButton, self).__init__(self.oDialog)
        sWorkingDir = oParent.get_working_dir()
        if sWorkingDir:
            self.set_current_folder(sWorkingDir)

    def add_filter_with_pattern(self, sName, aFilterPatterns):
        """Add a filter named sName to this widget, using the patterns
           in aFilterPatterns."""
        add_filter(self.oDialog, sName, aFilterPatterns)

    def default_filter(self):
        """Set the filter to be the default all files filter"""
        self.oDialog.default_filter()


class SimpleFileDialog(SutekhFileDialog):
    # pylint: disable-msg=R0904
    # gtk.Widget, so many public methods
    """A simple file dialog, which just returns the file name"""
    def __init__(self, oParent, sTitle, oAction):
        super(SimpleFileDialog, self).__init__(oParent, sTitle,
                oAction, (gtk.STOCK_OK, gtk.RESPONSE_OK,
                        gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
        self.connect("response", self.button_response)
        self.set_local_only(True)
        self.set_select_multiple(False)
        self.sName = None

    def button_response(self, _oWidget, iResponse):
        """Handle button press events"""
        if iResponse == gtk.RESPONSE_OK:
            self.sName = self.get_filename()
        self.destroy()

    def get_name(self):
        """Return the name to the caller."""
        return self.sName


class ImportDialog(SimpleFileDialog):
    # pylint: disable-msg=R0904
    # gtk.Widget, so many public methods
    """Prompt the user for a file name to import"""
    def __init__(self, sTitle, oParent):
        super(ImportDialog, self).__init__(oParent, sTitle,
                gtk.FILE_CHOOSER_ACTION_OPEN)


class ExportDialog(SimpleFileDialog):
    # pylint: disable-msg=R0904
    # gtk.Widget, so many public methods
    """Prompt the user for a filename to export to"""
    def __init__(self, sTitle, oParent, sDefaultFileName=None):
        super(ExportDialog, self).__init__(oParent, sTitle,
                gtk.FILE_CHOOSER_ACTION_SAVE)
        self.set_do_overwrite_confirmation(True)
        if sDefaultFileName:
            self.set_current_name(sDefaultFileName)


class ZipFileDialog(SimpleFileDialog):
    # pylint: disable-msg=R0904
    # gtk.Widget, so many public methods
    """Prompt the user for a zip file name"""
    def __init__(self, oParent, sTitle, oAction):
        super(ZipFileDialog, self).__init__(oParent, sTitle, oAction)
        self.add_filter_with_pattern('Zip Files', ['*.zip', '*.ZIP'])



# test_WriteLackeyCCG.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test Writing a card set to an Lackey CCG file"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.tests.core.test_PhysicalCardSet import make_set_1
from sutekh.core.CardSetHolder import CardSetWrapper
from sutekh.io.WriteLackeyCCG import  WriteLackeyCCG
import unittest

EXPECTED_1 = """4\t.44 Magnum
2\tAK-47
1\tAaron's Feeding Razor
2\tAbbot
2\tAbombwe
1\tPath of Blood, The
1\tScapelli, The Family 'Mechanic'
Crypt:
1\tAbebe
1\tAlan Sovereign Adv.
1\tInez ''Nurse216'' Villagrande
1\tSiamese, The
"""


class LackeyWriterTests(SutekhTest):
    """class for the Lackey CCG deck writer tests"""

    def test_deck_writer(self):
        """Test Lackey CCG deck writing"""
        oPhysCardSet1 = make_set_1()

        # Check output

        oWriter = WriteLackeyCCG()
        sData = self._round_trip_obj(oWriter, CardSetWrapper(oPhysCardSet1))

        self.assertEqual(sData, EXPECTED_1)


if __name__ == "__main__":
    unittest.main()



# CardSetController.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# Copyright 2006 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Controller for the card sets"""

from sqlobject import SQLObjectNotFound
from sutekh.gui.GuiCardSetFunctions import check_ok_to_delete, \
        update_card_set
from sutekh.gui.CardSetView import CardSetView
from sutekh.core.DBSignals import send_changed_signal
from sutekh.core.SutekhObjects import IPhysicalCardSet, PhysicalCardSet, \
        IAbstractCard, PhysicalCard, MapPhysicalCardToPhysicalCardSet, \
        IExpansion, IPhysicalCard
from sutekh.core.CardSetUtilities import delete_physical_card_set


class CardSetController(object):
    """Controller class for the Card Sets."""
    _sFilterType = 'PhysicalCard'

    def __init__(self, sName, oMainWindow, oFrame, bStartEditable):
        # pylint: disable-msg=E1101, E1103
        # SQLObject methods confuse pylint
        self._oMainWindow = oMainWindow
        self._oMenu = None
        self._oFrame = oFrame
        self._oView = CardSetView(oMainWindow, self, sName, bStartEditable)
        self.__oPhysCardSet = IPhysicalCardSet(sName)
        self.model.set_controller(self)

    # pylint: disable-msg=W0212
    # explicitly allow access to these values via thesep properties
    view = property(fget=lambda self: self._oView, doc="Associated View")
    model = property(fget=lambda self: self._oView._oModel, doc="View's Model")
    frame = property(fget=lambda self: self._oFrame, doc="Associated Frame")
    filtertype = property(fget=lambda self: self._sFilterType,
            doc="Associated Type")
    # pylint: enable-msg=W0212

    def cleanup(self):
        """Remove the signal handlers."""
        self.model.cleanup()

    def set_card_text(self, oCard):
        """Set card text to reflect selected card."""
        self._oMainWindow.set_card_text(oCard)

    def inc_card(self, oPhysCard, sCardSetName):
        """Returns True if a card was successfully added, False otherwise."""
        return self.add_card(oPhysCard, sCardSetName)

    def dec_card(self, oPhysCard, sCardSetName):
        """Returns True if a card was successfully removed, False otherwise."""
        # pylint: disable-msg=E1101, E1103
        # SQLObject +Pyprotocol methods confuse pylint
        try:
            if sCardSetName:
                oThePCS = IPhysicalCardSet(sCardSetName)
            else:
                oThePCS = self.__oPhysCardSet
            # find if the physical card given is in the set
            # Can only happen when expansion is None, since that may be a
            # top-level item or an expansion level item.
            # This also means, when removing cards from via a top-level item,
            # we will prefer removing cards without expansion information if
            # they are present.
            if not oPhysCard.expansion and \
                    MapPhysicalCardToPhysicalCardSet.selectBy(
                            physicalCardID=oPhysCard.id,
                            physicalCardSetID=oThePCS.id).count() == 0:
                # Given card is not in the card set, so consider all
                # cards with the same name.
                aPhysCards = list(PhysicalCard.selectBy(
                    abstractCardID=oPhysCard.abstractCard.id))
            else:
                aPhysCards = [oPhysCard]
        except SQLObjectNotFound:
            # Bail on error
            return False

        for oCard in aPhysCards:
            # Need to remove a single physical card from the mapping table
            # Can't use PhysicalCardSet.remove, as that removes all the cards
            aCandCards = list(MapPhysicalCardToPhysicalCardSet.selectBy(
                    physicalCardID=oCard.id,
                    physicalCardSetID=oThePCS.id))
            if len(aCandCards) > 0:
                # Found candidates, so remove last one
                MapPhysicalCardToPhysicalCardSet.delete(aCandCards[-1].id)
                oThePCS.syncUpdate()
                # signal to update the model
                send_changed_signal(oThePCS, oCard, -1)
                return True
        # Got here, so we failed to remove a card
        return False

    def add_card(self, oPhysCard, sCardSetName):
        """Returns True if a card was successfully added, False otherwise."""
        # pylint: disable-msg=E1101, E1103
        # SQLObject + PyProtocols methods confuse pylint
        try:
            if sCardSetName:
                oThePCS = IPhysicalCardSet(sCardSetName)
            else:
                oThePCS = self.__oPhysCardSet
        except SQLObjectNotFound:
            # Any error means we bail
            return False

        oThePCS.addPhysicalCard(oPhysCard.id)
        oThePCS.syncUpdate()
        # Signal to update the model
        send_changed_signal(oThePCS, oPhysCard, 1)
        return True

    def edit_properties(self, _oMenuWidget):
        """Run the dialog to update the card set properties"""
        update_card_set(self.__oPhysCardSet, self._oMainWindow)

    def update_to_new_db(self):
        """Update the internal card set to the new DB."""
        try:
            # cardset id -> profile map should have been updated before we
            # get here
            self.__oPhysCardSet = IPhysicalCardSet(self.view.sSetName)
            self.model.update_to_new_db(self.view.sSetName)
        except SQLObjectNotFound:
            # No longer in the database, so remove from the window
            self._oFrame.close_frame()

    def delete_card_set(self):
        """Delete this card set from the database."""
        # Check if CardSet is empty
        # pylint: disable-msg=E1101
        # sqlobject confuses pylint
        if check_ok_to_delete(self.__oPhysCardSet):
            self._oMainWindow.config_file.clear_cardset_profile(
                    self.model.cardset_id)  # Remove profile entry
            # Card Set is being deleted, so close pane
            self._oFrame.close_frame()
            # Close any other open copies as well
            for oFrame in self._oMainWindow.find_cs_pane_by_set_name(
                    self.view.sSetName):
                oFrame.close_frame()
            # Tell window to clean up
            delete_physical_card_set(self.view.sSetName)
            self._oMainWindow.reload_pcs_list()

    def save_iter_state(self, aIters):
        """Ask the view to save the state for us"""
        dStates = {}
        self.view.save_iter_state(aIters, dStates)
        return dStates

    def restore_iter_state(self, aIters, dStates):
        """Ask the view to restore the state"""
        self.view.restore_iter_state(aIters, dStates)

    def _get_card(self, sCardName, sExpansionName):
        """Convert card name & Expansion to PhsicalCard.

           Help for add_paste_data."""
        oExp = None
        try:
            if sExpansionName and sExpansionName != 'None' and \
                    sExpansionName != self.model.sUnknownExpansion:
                oExp = IExpansion(sExpansionName)
            oAbsCard = IAbstractCard(sCardName)
            oPhysCard = IPhysicalCard((oAbsCard, oExp))
            return oPhysCard
        except SQLObjectNotFound:
            # Error, so bail
            return None

    def add_paste_data(self, sSource, aCards):
        """Helper function for drag+drop and copy+paste.

           Only works when we're editable.
           """
        aSources = sSource.split(':')
        if not self.model.bEditable:
            return False
        if aSources[0] in ("Phys", PhysicalCardSet.sqlmeta.table):
            # Add the cards, Count Matters
            for iCount, sCardName, sExpansion in aCards:
                # Use None to indicate this card set
                oPhysCard = self._get_card(sCardName, sExpansion)
                if not oPhysCard:
                    # error, so skip this. (Warn user?)
                    continue
                if aSources[0] == "Phys":
                    # Only ever add 1 when dragging from physical card list
                    self.add_card(oPhysCard, None)
                else:
                    for _iLoop in range(iCount):
                        self.add_card(oPhysCard, None)
            return True
        else:
            return False

    def change_selected_card_count(self, dSelectedData):
        """Helper function to set the selected cards to the specified number"""
        for oPhysCard in dSelectedData:
            for sCardSetName, (iCardCount, iNewCnt) in \
                    dSelectedData[oPhysCard].iteritems():
                if iNewCnt < iCardCount:
                    # remove cards
                    for _iAttempt in range(iCardCount - iNewCnt):
                        # None as card set indicates this card set
                        self.dec_card(oPhysCard, sCardSetName)
                elif iNewCnt > iCardCount:
                    # add cards
                    for _iAttempt in range(iNewCnt - iCardCount):
                        self.inc_card(oPhysCard, sCardSetName)



# test_WriteArdbHTML.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test Writing a card set to an Ardb HTML file"""

import time
from sutekh.SutekhInfo import SutekhInfo
from sutekh.tests.TestCore import SutekhTest
from sutekh.tests.core.test_PhysicalCardSet import make_set_1
from sutekh.core.CardSetHolder import CardSetWrapper
from sutekh.io.WriteArdbHTML import  WriteArdbHTML, HTML_STYLE
import unittest

EXPECTED_1 = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html lang="en" xml:lang="en" xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta content="text/html; charset=&quot;us-ascii&quot;" http-equiv="content-type" />
    <style type="text/css">%s</style>
    <title>VTES deck : Test Set 1 by A test author</title>
  </head><body>
    <div id="info">
      <h1 id="nametitle">
        <span>Deck Name :</span>
        <span class="value" id="namevalue">Test Set 1</span>
      </h1><h2 id="authortitle">
        <span>Author : </span>
        <span class="value" id="authornamevalue">A test author</span>
      </h2><h2 id="description">
        <span>Description : </span>
      </h2><p>
        <span class="value" id="descriptionvalue">A test comment</span>
      </p>
    </div><div id="crypt">
      <h3 id="crypttitle">
        <span>Crypt</span>
        <span>[4 vampires] Capacity min : 3 max : 7 average : 5.00</span>
      </h3><div id="crypttable">
        <table summary="Crypt card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showvamp.html?NAME=Siamese,%%20The">The Siamese</a>
                </span>
              </td><td />
              <td>
                <span class="tablevalue">7</span>
              </td><td>
                <span class="tablevalue">PRE SPI ani pro</span>
              </td><td />
              <td>
                <span class="tablevalue">Ahrimane (group 2)</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showvamp.html?NAME=Alan%%20Sovereign%%20ADV">Alan Sovereign</a>
                </span>
              </td><td>
                <span class="tablevalue">(Advanced)</span>
              </td><td>
                <span class="tablevalue">6</span>
              </td><td>
                <span class="tablevalue">AUS DOM for pre</span>
              </td><td />
              <td>
                <span class="tablevalue">Ventrue (group 3)</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showvamp.html?NAME=Abebe">Abebe</a>
                </span>
              </td><td />
              <td>
                <span class="tablevalue">4</span>
              </td><td>
                <span class="tablevalue">nec obf thn</span>
              </td><td />
              <td>
                <span class="tablevalue">Samedi (group 4)</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showvamp.html?NAME=Inez%%20&quot;Nurse216&quot;%%20Villagrande">Inez "Nurse216" Villagrande</a>
                </span>
              </td><td />
              <td>
                <span class="tablevalue">3</span>
              </td><td>
                <span class="tablevalue">inn</span>
              </td><td />
              <td>
                <span class="tablevalue">Innocent (Imbued) (group 4)</span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div id="library">
      <h3 id="librarytitle">
        <span>Library</span>
        <span class="stats" id="librarystats">[13 cards]</span>
      </h3><div class="librarytable">
        <h4 class="librarytype">
          <span>Action</span>
          <span class="stats">[2]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">2x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showcard.html?NAME=Abbot">Abbot</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Ally</span>
          <span class="stats">[1]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showcard.html?NAME=Scapelli,%%20The%%20Family%%20&quot;Mechanic&quot;">Scapelli, The Family "Mechanic"</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Equipment</span>
          <span class="stats">[7]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">4x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showcard.html?NAME=.44%%20Magnum">.44 Magnum</a>
                </span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">2x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showcard.html?NAME=AK-47">AK-47</a>
                </span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showcard.html?NAME=Aaron's%%20Feeding%%20Razor">Aaron's Feeding Razor</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Master</span>
          <span class="stats">[3]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">2x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showcard.html?NAME=Abombwe">Abombwe</a>
                </span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://monger.vekn.org/showcard.html?NAME=Path%%20of%%20Blood,%%20The">The Path of Blood</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div>
      <span class="generator">Crafted with : Sutekh [ %s ]. [ DATE ]</span>
    </div>
  </body>
</html>""" % (HTML_STYLE, SutekhInfo.VERSION_STR)

EXPECTED_2 = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html lang="en" xml:lang="en" xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta content="text/html; charset=&quot;us-ascii&quot;" http-equiv="content-type" />
    <style type="text/css">%s</style>
    <title>VTES deck : Test Set 1 by A test author</title>
  </head><body>
    <div id="info">
      <h1 id="nametitle">
        <span>Deck Name :</span>
        <span class="value" id="namevalue">Test Set 1</span>
      </h1><h2 id="authortitle">
        <span>Author : </span>
        <span class="value" id="authornamevalue">A test author</span>
      </h2><h2 id="description">
        <span>Description : </span>
      </h2><p>
        <span class="value" id="descriptionvalue">A test comment</span>
      </p>
    </div><div id="crypt">
      <h3 id="crypttitle">
        <span>Crypt</span>
        <span>[4 vampires] Capacity min : 3 max : 7 average : 5.00</span>
      </h3><div id="crypttable">
        <table summary="Crypt card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">The Siamese</span>
              </td><td />
              <td>
                <span class="tablevalue">7</span>
              </td><td>
                <span class="tablevalue">PRE SPI ani pro</span>
              </td><td />
              <td>
                <span class="tablevalue">Ahrimane (group 2)</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">Alan Sovereign</span>
              </td><td>
                <span class="tablevalue">(Advanced)</span>
              </td><td>
                <span class="tablevalue">6</span>
              </td><td>
                <span class="tablevalue">AUS DOM for pre</span>
              </td><td />
              <td>
                <span class="tablevalue">Ventrue (group 3)</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">Abebe</span>
              </td><td />
              <td>
                <span class="tablevalue">4</span>
              </td><td>
                <span class="tablevalue">nec obf thn</span>
              </td><td />
              <td>
                <span class="tablevalue">Samedi (group 4)</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">Inez "Nurse216" Villagrande</span>
              </td><td />
              <td>
                <span class="tablevalue">3</span>
              </td><td>
                <span class="tablevalue">inn</span>
              </td><td />
              <td>
                <span class="tablevalue">Innocent (Imbued) (group 4)</span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div id="library">
      <h3 id="librarytitle">
        <span>Library</span>
        <span class="stats" id="librarystats">[13 cards]</span>
      </h3><div class="librarytable">
        <h4 class="librarytype">
          <span>Action</span>
          <span class="stats">[2]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">2x</span>
              </td><td>
                <span class="tablevalue">Abbot</span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Ally</span>
          <span class="stats">[1]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">Scapelli, The Family "Mechanic"</span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Equipment</span>
          <span class="stats">[7]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">4x</span>
              </td><td>
                <span class="tablevalue">.44 Magnum</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">2x</span>
              </td><td>
                <span class="tablevalue">AK-47</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">Aaron's Feeding Razor</span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Master</span>
          <span class="stats">[3]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">2x</span>
              </td><td>
                <span class="tablevalue">Abombwe</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">The Path of Blood</span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div>
      <span class="generator">Crafted with : Sutekh [ %s ]. [ DATE ]</span>
    </div>
  </body>
</html>""" % (HTML_STYLE, SutekhInfo.VERSION_STR)

EXPECTED_3 = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html lang="en" xml:lang="en" xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta content="text/html; charset=&quot;us-ascii&quot;" http-equiv="content-type" />
    <style type="text/css">%s</style>
    <title>VTES deck : Test Set 1 by A test author</title>
  </head><body>
    <div id="info">
      <h1 id="nametitle">
        <span>Deck Name :</span>
        <span class="value" id="namevalue">Test Set 1</span>
      </h1><h2 id="authortitle">
        <span>Author : </span>
        <span class="value" id="authornamevalue">A test author</span>
      </h2><h2 id="description">
        <span>Description : </span>
      </h2><p>
        <span class="value" id="descriptionvalue">A test comment</span>
      </p>
    </div><div id="crypt">
      <h3 id="crypttitle">
        <span>Crypt</span>
        <span>[4 vampires] Capacity min : 3 max : 7 average : 5.00</span>
      </h3><div id="crypttable">
        <table summary="Crypt card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?crypt=Siamese,+The">The Siamese</a>
                </span>
              </td><td />
              <td>
                <span class="tablevalue">7</span>
              </td><td>
                <span class="tablevalue">PRE SPI ani pro</span>
              </td><td />
              <td>
                <span class="tablevalue">Ahrimane (group 2)</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?crypt=Alan+Sovereign+Adv">Alan Sovereign</a>
                </span>
              </td><td>
                <span class="tablevalue">(Advanced)</span>
              </td><td>
                <span class="tablevalue">6</span>
              </td><td>
                <span class="tablevalue">AUS DOM for pre</span>
              </td><td />
              <td>
                <span class="tablevalue">Ventrue (group 3)</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?crypt=Abebe">Abebe</a>
                </span>
              </td><td />
              <td>
                <span class="tablevalue">4</span>
              </td><td>
                <span class="tablevalue">nec obf thn</span>
              </td><td />
              <td>
                <span class="tablevalue">Samedi (group 4)</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?crypt=Inez+Nurse216+Villagrande">Inez "Nurse216" Villagrande</a>
                </span>
              </td><td />
              <td>
                <span class="tablevalue">3</span>
              </td><td>
                <span class="tablevalue">inn</span>
              </td><td />
              <td>
                <span class="tablevalue">Innocent (Imbued) (group 4)</span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div id="library">
      <h3 id="librarytitle">
        <span>Library</span>
        <span class="stats" id="librarystats">[13 cards]</span>
      </h3><div class="librarytable">
        <h4 class="librarytype">
          <span>Action</span>
          <span class="stats">[2]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">2x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=Abbot">Abbot</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Ally</span>
          <span class="stats">[1]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=Scapelli,+The+Family+Mechanic">Scapelli, The Family "Mechanic"</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Equipment</span>
          <span class="stats">[7]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">4x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=.44+Magnum">.44 Magnum</a>
                </span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">2x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=AK-47">AK-47</a>
                </span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=Aaron's+Feeding+Razor">Aaron's Feeding Razor</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Master</span>
          <span class="stats">[3]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">2x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=Abombwe">Abombwe</a>
                </span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=Path+of+Blood,+The">The Path of Blood</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div>
      <span class="generator">Crafted with : Sutekh [ %s ]. [ DATE ]</span>
    </div>
  </body>
</html>""" % (HTML_STYLE, SutekhInfo.VERSION_STR)

EXPECTED_4 = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html lang="en" xml:lang="en" xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta content="text/html; charset=&quot;us-ascii&quot;" http-equiv="content-type" />
    <style type="text/css">%s</style>
    <title>VTES deck : Test Set 1 by A test author</title>
  </head><body>
    <div id="info">
      <h1 id="nametitle">
        <span>Deck Name :</span>
        <span class="value" id="namevalue">Test Set 1</span>
      </h1><h2 id="authortitle">
        <span>Author : </span>
        <span class="value" id="authornamevalue">A test author</span>
      </h2><h2 id="description">
        <span>Description : </span>
      </h2><p>
        <span class="value" id="descriptionvalue">A test comment</span>
      </p>
    </div><div id="crypt">
      <h3 id="crypttitle">
        <span>Crypt</span>
        <span>[4 vampires] Capacity min : 3 max : 7 average : 5.00</span>
      </h3><div id="crypttable">
        <table summary="Crypt card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?crypt=Siamese,+The">The Siamese</a>
                </span>
              </td><td />
              <td>
                <span class="tablevalue">7</span>
              </td><td>
                <span class="tablevalue">PRE SPI ani pro</span>
              </td><td />
              <td>
                <span class="tablevalue">Ahrimane (group 2)</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?crypt=Alan+Sovereign+Adv">Alan Sovereign</a>
                </span>
              </td><td>
                <span class="tablevalue">(Advanced)</span>
              </td><td>
                <span class="tablevalue">6</span>
              </td><td>
                <span class="tablevalue">AUS DOM for pre</span>
              </td><td />
              <td>
                <span class="tablevalue">Ventrue (group 3)</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?crypt=Abebe">Abebe</a>
                </span>
              </td><td />
              <td>
                <span class="tablevalue">4</span>
              </td><td>
                <span class="tablevalue">nec obf thn</span>
              </td><td />
              <td>
                <span class="tablevalue">Samedi (group 4)</span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?crypt=Inez+Nurse216+Villagrande">Inez "Nurse216" Villagrande</a>
                </span>
              </td><td />
              <td>
                <span class="tablevalue">3</span>
              </td><td>
                <span class="tablevalue">inn</span>
              </td><td />
              <td>
                <span class="tablevalue">Innocent (Imbued) (group 4)</span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div id="library">
      <h3 id="librarytitle">
        <span>Library</span>
        <span class="stats" id="librarystats">[13 cards]</span>
      </h3><div class="librarytable">
        <h4 class="librarytype">
          <span>Action</span>
          <span class="stats">[2]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">2x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=Abbot">Abbot</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Ally</span>
          <span class="stats">[1]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=Scapelli,+The+Family+Mechanic">Scapelli, The Family "Mechanic"</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Equipment</span>
          <span class="stats">[7]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">4x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=.44+Magnum">.44 Magnum</a>
                </span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">2x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=AK-47">AK-47</a>
                </span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=Aaron's+Feeding+Razor">Aaron's Feeding Razor</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table><h4 class="librarytype">
          <span>Master</span>
          <span class="stats">[3]</span>
        </h4><table summary="Library card table">
          <tbody>
            <tr>
              <td>
                <span class="tablevalue">2x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=Abombwe">Abombwe</a>
                </span>
              </td>
            </tr><tr>
              <td>
                <span class="tablevalue">1x</span>
              </td><td>
                <span class="tablevalue">
                  <a href="http://www.secretlibrary.info/?lib=Path+of+Blood,+The">The Path of Blood</a>
                </span>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div><div id="cardtext">
      <h3 class="cardtext">
        <span>Card Texts</span>
      </h3><h4 class="librarytype">Crypt</h4>
      <h5>The Siamese</h5>
      <ul>
        <li>
          <span class="label">Capacity:</span>
          <span class="capacity">7</span>
        </li><li>
          <span class="label">Group:</span>
          <span class="group">2</span>
        </li><li>
          <span class="label">Clan:</span>
          <span class="clan">Ahrimane</span>
        </li><li>
          <span class="label">Disciplines:</span>
          <span class="disciplines">PRE SPI ani pro</span>
        </li>
      </ul><div class="text">
        <p>Sabbat: +1 bleed. Sterile.</p>
      </div><h5>Alan Sovereign (Advanced)</h5>
      <ul>
        <li>
          <span class="label">Capacity:</span>
          <span class="capacity">6</span>
        </li><li>
          <span class="label">Group:</span>
          <span class="group">3</span>
        </li><li>
          <span class="label">Clan:</span>
          <span class="clan">Ventrue</span>
        </li><li>
          <span class="label">Disciplines:</span>
          <span class="disciplines">AUS DOM for pre</span>
        </li>
      </ul><div class="text">
        <p>Advanced, Camarilla: While Alan is ready, you may pay some or all of the pool cost of equipping from any investment cards you control.</p>
        <p>[MERGED] During your master phase, if Alan is ready, you may move a counter from any investment card to your pool.</p>
      </div><h5>Abebe</h5>
      <ul>
        <li>
          <span class="label">Capacity:</span>
          <span class="capacity">4</span>
        </li><li>
          <span class="label">Group:</span>
          <span class="group">4</span>
        </li><li>
          <span class="label">Clan:</span>
          <span class="clan">Samedi</span>
        </li><li>
          <span class="label">Disciplines:</span>
          <span class="disciplines">nec obf thn</span>
        </li>
      </ul><div class="text">
        <p>Independent.</p>
      </div><h5>Inez "Nurse216" Villagrande</h5>
      <ul>
        <li>
          <span class="label">Capacity:</span>
          <span class="capacity">3</span>
        </li><li>
          <span class="label">Group:</span>
          <span class="group">4</span>
        </li><li>
          <span class="label">Clan:</span>
          <span class="clan">Innocent (Imbued)</span>
        </li><li>
          <span class="label">Disciplines:</span>
          <span class="disciplines">inn</span>
        </li>
      </ul><div class="text">
        <p>When Inez enters play, you may search your library (shuffle afterward) or hand for a power that requires innocence and put it on her.</p>
      </div><h4 class="libraryttype">Action</h4>
      <h5 class="cardname">Abbot</h5>
      <div class="text">
        <p>+1 stealth action. Requires a Sabbat vampire.</p>
        <p>Put this card on this acting Sabbat vampire and untap him or her. This Sabbat vampire gets +1 intercept against (D) actions directed at his or her controller. A vampire may have only one Abbot.</p>
      </div><h4 class="libraryttype">Ally</h4>
      <h5 class="cardname">Scapelli, The Family "Mechanic"</h5>
      <ul>
        <li>
          <span class="label">Requires:</span>
          <span class="requirement">Giovanni</span>
        </li><li>
          <span class="label">Cost:</span>
          <span class="cost">3 pool</span>
        </li>
      </ul><div class="text">
        <p>Unique {mortal} with 3 life. {0 strength}, 1 bleed.</p>
        <p>{Scapelli may strike for 2R damage.} Once each combat, Scapelli may press to continue combat.</p>
      </div><h4 class="libraryttype">Equipment</h4>
      <h5 class="cardname">.44 Magnum</h5>
      <ul>
        <li>
          <span class="label">Cost:</span>
          <span class="cost">2 pool</span>
        </li>
      </ul><div class="text">
        <p>Weapon, gun.</p>
        <p>2R damage each strike, with an optional maneuver each combat.</p>
      </div><h5 class="cardname">AK-47</h5>
      <ul>
        <li>
          <span class="label">Cost:</span>
          <span class="cost">5 pool</span>
        </li>
      </ul><div class="text">
        <p>Weapon. Gun.</p>
        <p>2R damage each strike, with an optional maneuver {each combat}. When bearer strikes with this gun, he or she gets an optional additional strike this round, only usable to strike with this gun.</p>
      </div><h5 class="cardname">Aaron's Feeding Razor</h5>
      <ul>
        <li>
          <span class="label">Cost:</span>
          <span class="cost">1 pool</span>
        </li>
      </ul><div class="text">
        <p>Unique equipment.</p>
        <p>When this vampire successfully hunts, he or she gains 1 additional blood.</p>
      </div><h4 class="libraryttype">Master</h4>
      <h5 class="cardname">Abombwe</h5>
      <div class="text">
        <p>Master: Discipline. Trifle.</p>
        <p>Put this card on a Laibon or on a vampire with Protean [pro]. This vampire gains one level of Abombwe [abo]. Capacity increases by 1: the vampire is one generation older. Cannot be placed on a vampire with superior Abombwe.</p>
      </div><h5 class="cardname">The Path of Blood</h5>
      <ul>
        <li>
          <span class="label">Requires:</span>
          <span class="requirement">Assamite</span>
        </li><li>
          <span class="label">Cost:</span>
          <span class="cost">1 pool</span>
        </li>
      </ul><div class="text">
        <p>Unique master.</p>
        <p>Put this card in play. Cards that require Quietus [qui] {cost Assamites 1 less blood}. Any minion may burn this card as a (D) action; if that minion is a vampire, he or she then takes 1 unpreventable damage when this card is burned.</p>
      </div>
    </div><div>
      <span class="generator">Crafted with : Sutekh [ %s ]. [ DATE ]</span>
    </div>
  </body>
</html>""" % (HTML_STYLE, SutekhInfo.VERSION_STR)


class ARDBHTMLWriterTests(SutekhTest):
    """class for the ARDB HTML deck writer tests"""

    def test_deck_writer(self):
        """Test HTML deck writing"""
        oPhysCardSet1 = make_set_1()

        sCurDate = time.strftime('[ %Y-%m-%d ]', time.localtime())
        # Check output

        oWriter = WriteArdbHTML(bDoText=False)
        sData = self._round_trip_obj(oWriter, CardSetWrapper(oPhysCardSet1))
        sData = sData.replace(sCurDate, '[ DATE ]')

        self.assertEqual(sData, EXPECTED_1)

        # Test other modes
        oWriter = WriteArdbHTML('Monger', False)
        sData = self._round_trip_obj(oWriter, CardSetWrapper(oPhysCardSet1))
        sData = sData.replace(sCurDate, '[ DATE ]')

        self.assertEqual(sData, EXPECTED_1)

        oWriter = WriteArdbHTML('None', False)
        sData = self._round_trip_obj(oWriter, CardSetWrapper(oPhysCardSet1))
        sData = sData.replace(sCurDate, '[ DATE ]')

        self.assertEqual(sData, EXPECTED_2)

        oWriter = WriteArdbHTML('Secret Library', False)
        sData = self._round_trip_obj(oWriter, CardSetWrapper(oPhysCardSet1))
        sData = sData.replace(sCurDate, '[ DATE ]')

        self.assertEqual(sData, EXPECTED_3)

        oWriter = WriteArdbHTML('Secret Library', True)
        sData = self._round_trip_obj(oWriter, CardSetWrapper(oPhysCardSet1))
        sData = sData.replace(sCurDate, '[ DATE ]')

        self.assertEqual(sData, EXPECTED_4)


if __name__ == "__main__":
    unittest.main()



# CardListTabulator.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Create a table (as a list of list) from a list of cards"""

from sutekh.core.SutekhObjects import Discipline, Clan, Rarity, Expansion, \
        CardType, IAbstractCard


class CardListTabulator(object):
    """Creates a table of cards from a card list.

       Each row of the table corresponds to a card in the list.
       Each column of the table represents a numerical property of the card
       (pool cost, blood cost, discipline levels etc).
       """

    def __init__(self, aColNames, dPropFuncs):
        """Create the Table

           aColNames: Keys from dPropFuncs in the order in which you want the
                      table columns to appear.
           dPropFuncs: Dictionary of functions for finding properties. Keys are
                       strings suitable for use as column headings,
                       values are functions f(oAbstractCard) -> int which
                       return the column value given an abstract card object.
           """
        self._aColNames = aColNames
        self._dPropFuncs = dPropFuncs

    @staticmethod
    def get_default_prop_funcs():
        """Return a dictionary of card property creation functions.

           The keys are strings suitable for use as column names, the values
           are functions for calculating a column entry from an abstract card,
           i.e. f: AbstractCard -> Int.  Since a new dictionary is created
           every time this function is called it's safe to edit it to include
           just the functions you want.

           Notes:
             * For blood, pool and conviction costs, a cost of X is represented
               by a value of -1.
             * Boolean attributes (disciplines, rarities, expansions, clans and
               cardtypes) are represented by values of 0 or 1.
             * Group and capacity are set to 0 if the card doesn't have a group
               or capacity.
           """
        dProps = {}

        # properties from direct attributes of Abstract Cards
        dProps['group'] = lambda card: (card.group is not None and
                card.group) or 0
        dProps['capacity'] = lambda card: (card.capacity is not None and
                card.capacity) or 0
        dProps['pool cost'] = lambda card: (card.costtype == 'pool' and
                card.cost) or 0
        dProps['blood cost'] = lambda card: (card.costtype == 'blood' and
                card.cost) or 0
        dProps['conviction cost'] = lambda card: (card.costtype == 'conviction'
                and card.cost) or 0
        dProps['advanced'] = lambda card: (card.level == 'advanced' and
                1) or 0
        dProps['physical card count'] = lambda card: len(card.physicalCards)

        # The little helper functions below are necessary for scoping reasons.

        # discipline properties
        def make_dis_func(oTmpDis):
            """Create a function that tests for the given discipline."""
            return lambda card: ((oTmpDis in [oPair.discipline for oPair in
                card.discipline]) and 1) or 0

        for oDis in Discipline.select():
            dProps['discipline: ' + oDis.fullname] = make_dis_func(oDis)

        # rarity properties
        def make_rar_func(oTmpRar):
            """Create a function that tests for the given rarity."""
            return lambda card: ((oTmpRar in [oPair.rarity for oPair in
                card.rarity]) and 1) or 0

        for oRar in Rarity.select():
            dProps['rarity: ' + oRar.name] = make_rar_func(oRar)

        # expansion properties
        def make_exp_func(oTmpExp):
            """Create a function that tests for the given expansion."""
            return lambda card: ((oTmpExp in [oPair.expansion for oPair in
                card.rarity]) and 1) or 0

        for oExp in Expansion.select():
            dProps['expansion: ' + oExp.name] = make_exp_func(oExp)

        # clan properties
        def make_clan_func(oTmpClan):
            """Create a function that tests for the given clan"""
            return lambda card: ((oTmpClan in card.clan) and 1) or 0

        for oClan in Clan.select():
            dProps['clan: ' + oClan.name] = make_clan_func(oClan)

        # cardtype properties
        def make_card_type_func(oTmpType):
            """Create a function that tests for the given card type."""
            return lambda card: ((oTmpType in card.cardtype) and 1) or 0

        for oType in CardType.select():
            dProps['card type: ' + oType.name] = make_card_type_func(oType)

        return dProps

    def tabulate(self, aCards):
        """Create a table from the list of (or iterator over) cards.

           Returns the table which is a nested list where each element is a row
           and each row consists of column values.
           The rows are in the same order as the cards in aCards.
           The columns are in the same order as in the aColNames list passed to
           __init__.
           """
        aColFuncs = [self._dPropFuncs[x] for x in self._aColNames]

        aTable = []

        for oCard in aCards:
            oCard = IAbstractCard(oCard)
            aRow = []

            for fProp in aColFuncs:
                aRow.append(fProp(oCard))

            aTable.append(aRow)

        return aTable



# textile2html.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""
Convert Sutekh textile documentation into HTML pages.
"""

import glob
import os
import textile

# pylint: disable-msg=C0103
# we ignore our usual naming conventions here

try:
    import tidy
except ImportError:
    tidy = None

HTML_HEADER = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html lang="en" xml:lang="en" xmlns="http://www.w3.org/1999/xhtml">
<head>
    <title>%(title)s</title>
</head>
<body>
"""

HTML_FOOTER = """
</body>
</html>
"""

# pylint: enable-msg=C0103

# TODO: Pull in version number automatically from SutekhInfo


def textile2html(sText, dContext):
    """Convert a Textile markup string to an HTML file."""
    sHtml = HTML_HEADER % dContext \
        + textile.textile(sText) \
        + HTML_FOOTER % dContext
    sHtml = sHtml.replace('<br />', ' ')  # remove pesky linebreaks
    return sHtml


def convert(sTextileDir, sHtmlDir):
    """Convert all .txt files in sTextileDir to .html files in sHtmlDir."""
    for sTextilePath in glob.glob(os.path.join(sTextileDir, "*.txt")):
        sBasename = os.path.basename(sTextilePath)
        sFilename, _sExt = os.path.splitext(sBasename)
        sHtmlPath = os.path.join(sHtmlDir, sFilename + ".html")

        dContext = {
            'title': "Sutekh " + sFilename,
        }

        fTextile = file(sTextilePath, "rb")
        fHtml = file(sHtmlPath, "wb")

        # NB: Late night fast 'n dirty hack alert
        # Annoyingly, python-textile 2.1 doesn't handle list elements split
        # over multiple lines the way python-textile 2.0 does [1], so we need
        # to manually join lines before feeding them to textile
        # We use the tradional trailing \ to indicate continuation
        # [1] Note that the 2.10 version in karmic is a misnumbered 2.0.10
        aLines = []
        aCurLine = []
        for sLine in fTextile.readlines():
            if sLine.endswith("\\\n"):
                if aCurLine:
                    aCurLine.append(sLine[:-2])
                else:
                    aCurLine = [sLine[:-2]]
                continue
            elif aCurLine:
                aCurLine.append(sLine)
                aLines.append(''.join(aCurLine))
                aCurLine = []
            else:
                aLines.append(sLine)

        fHtml.write(textile2html(''.join(aLines), dContext))

        fTextile.close()
        fHtml.close()

        if tidy is not None:
            aErrors = tidy.parse(sHtmlPath).get_errors()
            if aErrors:
                print 'tidy reports the following errors for %s' % sHtmlPath
                print '\n'.join([x.err for x in aErrors])


if __name__ == "__main__":
    convert("textile", "html")



# WriteArdbText.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Writer for ther ARDB text deck format.

   Example deck:

   Deck Name : Followers of Set Preconstructed Deck
   Author : L. Scott Johnson
   Description :
   Followers of Set Preconstructed Starter from Lords of the Night.

   http://www.white-wolf.com/vtes/index.php?line=Checklist_LordsOfTheNight

   Crypt [12 vampires] Capacity min: 2 max: 10 average: 5.84
   ------------------------------------------------------------
   2x Nakhthorheb			  10 OBF PRE SER           Follower :4
   ...

   Library [77 cards]
   ------------------------------------------------------------
   Action [20]
     2x Blithe Acceptance
     4x Dream World
   ...
   """

from sutekh.core.ArdbInfo import ArdbInfo


class WriteArdbText(ArdbInfo):
    """Create a string in ARDB's text format representing a dictionary
       of cards."""

    # pylint: disable-msg=R0201
    # method for consistency with the other methods
    def _gen_header(self, oHolder):
        """Generate an ARDB text file header."""
        return "Deck Name : %s\n" \
               "Author : %s\n" \
               "Description :\n%s\n" % (oHolder.name, oHolder.author,
                       oHolder.comment)
    # pylint: enable-msg=R0201

    def _gen_crypt(self, dCards):
        """Generaten an ARDB text file crypt description.

           dCards is mapping of (card id, card name) -> card count.
           """
        (dVamps, dCryptStats) = self._extract_crypt(dCards)
        dCombinedVamps = self._group_sets(dVamps)

        sCrypt = "Crypt [%(size)d vampires] Capacity min: %(min)d " \
                "max: %(max)d average: %(avg).2f\n" \
                "-------------------------------------------------" \
                "-----------\n" \
                % dCryptStats

        aCryptLines = []
        # ARDB's discipline & title padding are based on the longest entry
        # so we need to keep track and format later
        iDiscJust = 0
        iTitleJust = 0
        for oCard,  (iCount, _sSet) in sorted(dCombinedVamps.iteritems(),
                key=lambda x: (-x[1][0], -max(x[0].capacity, x[0].life),
                    x[0].name)):
            # We sort inversely on count, then capacity and then normally by
            # name
            dLine = {'count': iCount}
            if len(oCard.creed) > 0:
                dLine['clan'] = "%s (Imbued)" % \
                        [x.name for x in oCard.creed][0]
                dLine['capacity'] = oCard.life
            else:
                dLine['clan'] = [x.name for x in oCard.clan][0]
                if dLine['clan'].endswith('antitribu'):
                    # ARDB doesn't truncate antitribu further
                    dLine['clan'] = '!' + dLine['clan'].replace(' antitribu',
                            '')
                else:
                    dLine['clan'] = dLine['clan'][:10]  # truncate if needed
                dLine['capacity'] = oCard.capacity
            dLine['name'] = oCard.name
            dLine['adv'] = '   '
            if oCard.level is not None:
                dLine['name'] = dLine['name'].replace(' (Advanced)', '')
                dLine['adv'] = 'Adv'
            dLine['name'] = dLine['name'].ljust(18)[:18]  # truncate if needed
            dLine['disc'] = self._gen_disciplines(oCard)
            iDiscJust = max(iDiscJust, len(dLine['disc']))

            dLine['title'] = '   '
            if len(oCard.title):
                dLine['title'] = [oC.name for oC in oCard.title][0]
                dLine['title'] = dLine['title'].replace('Independent with ',
                        '')[:12]
                iTitleJust = max(iTitleJust, len(dLine['title']))
            dLine['group'] = int(oCard.group)
            aCryptLines.append(dLine)

        for dLine in aCryptLines:
            dLine['title'] = dLine['title'].ljust(iTitleJust)
            dLine['disc'] = dLine['disc'].ljust(iDiscJust)
            sCrypt += " %(count)dx %(name)s %(adv)s %(capacity)d" \
                    " %(disc)s %(title)s %(clan)s:%(group)d\n" % dLine

        return sCrypt

    def _gen_library(self, dCards):
        """Generaten an ARDB text file library description.

           dCards is mapping of (card id, card name) -> card count.
           """
        (dLib, iLibSize) = self._extract_library(dCards)
        dCombinedLib = self._group_sets(dLib)

        sLib = "Library [%d cards]\n" \
            "------------------------------------------------------------\n" \
            % (iLibSize,)

        dTypes = {}
        for oCard, (iCount, sTypeString, _sSet) in dCombinedLib.iteritems():
            if sTypeString not in dTypes:
                dTypes[sTypeString] = {}
            dTypes[sTypeString].setdefault(oCard, 0)
            dTypes[sTypeString][oCard] += iCount

        for sTypeString in sorted(dTypes):
            dCards = dTypes[sTypeString]
            iTotal = sum(dCards.values())

            sLib += "%s [%d]\n" % (sTypeString, iTotal)

            for oCard, iCount in sorted(dCards.iteritems(),
                    key=lambda x: x[0].name):
                sLib += " %dx %s\n" % (iCount, oCard.name)

            sLib += "\n"

        return sLib

    # pylint: disable-msg=R0913
    # we need all these arguments
    def write(self, fOut, oHolder):
        """Takes filename, deck details and a dictionary of cards, of the
           form dCard[(id, name, set)] = count and writes the file."""
        # We don't encode strange characters, and just write the unicode string
        # to file. This looks to match ARDB's behaviour (tested against
        # ARDB version 2.8
        dCards = self._get_cards(oHolder.cards)
        fOut.write(self._gen_header(oHolder))
        fOut.write("\n")
        fOut.write(self._gen_crypt(dCards))
        fOut.write("\n")
        fOut.write(self._gen_library(dCards))

    # pylint: enable-msg=R0913



# Rulebook.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Simon Cross <hodgestar+sutekh@gmail.com>
# GPL - see COPYING for details

"""Downloads rulebook HTML pages and makes them available via the Help menu."""

from sutekh.io.DataPack import find_data_pack, DOC_URL
from sutekh.gui.PluginManager import SutekhPlugin
from sutekh.gui.FileOrUrlWidget import FileOrUrlWidget
from sutekh.gui.SutekhDialog import SutekhDialog, do_exception_complaint
from sutekh.gui.FileOrUrlWidget import fetch_data
from sutekh.gui.SutekhFileWidget import add_filter
from sutekh.gui.ProgressDialog import ProgressDialog, SutekhCountLogHandler
from sutekh.SutekhUtility import prefs_dir, ensure_dir_exists
import gtk
import os
import StringIO
import urllib2
import webbrowser
import zipfile
from logging import Logger


class RulebookConfigDialog(SutekhDialog):
    # pylint: disable-msg=R0904
    # R0904 - gtk Widget, so has many public methods
    """Dialog for configuring the Rulebook plugin."""

    sDocUrl = DOC_URL

    def __init__(self, oParent, bFirstTime=False):
        super(RulebookConfigDialog, self).__init__(
                'Configure Rulebook Info Plugin', oParent,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL,
                    gtk.RESPONSE_CANCEL))
        oDescLabel = gtk.Label()
        if not bFirstTime:
            oDescLabel.set_markup('<b>Choose how to configure the rulebook'
                    ' plugin</b>')
        else:
            oDescLabel.set_markup('<b>Choose how to configure the rulebook'
                    ' plugin</b>\nChoose cancel to skip configuring the '
                    ' plugin\nYou will not be prompted again')
        self.oFileWidget = FileOrUrlWidget(oParent, "Choose location for "
                "rulebook zip file", {'Sutekh Wiki': self.sDocUrl})
        add_filter(self.oFileWidget, 'Zip Files', ['*.zip', '*.ZIP'])
        # pylint: disable-msg=E1101
        # pylint doesn't pick up vbox methods correctly
        self.vbox.pack_start(oDescLabel, False, False)
        self.vbox.pack_start(self.oFileWidget, False, False)
        if bFirstTime:
            # description is longer, so request should be taller
            self.set_size_request(340, 240)
        else:
            self.set_size_request(340, 200)

        self.show_all()

    def get_data(self):
        """Return the zip file data containing the rulebooks"""
        sFile, _bUrl = self.oFileWidget.get_file_or_url()
        sData = None
        if sFile == self.sDocUrl:
            # Downloading from sutekh wiki, so need magic to get right file
            sZipUrl = find_data_pack('rulebooks')
            oFile = urllib2.urlopen(sZipUrl)
            sData = fetch_data(oFile)
        elif sFile:
            sData = self.oFileWidget.get_binary_data()
        return sData


class RulebookPlugin(SutekhPlugin):
    """Plugin allowing downloading of rulebook HTML pages and making them
       available via the Plugins menu.
       """
    aModelsSupported = ("MainWindow",)

    dGlobalConfig = {
        'rulebook path': 'string(default=None)',
    }

    # pylint: disable-msg=W0142
    # ** magic OK here
    def __init__(self, *args, **kwargs):
        super(RulebookPlugin, self).__init__(*args, **kwargs)
        self._oRulebookDlg = None
        # TODO: First menu item is a hackish way to get at the parent Rulebook
        #       menu. Remove this once the plugin API provides a better way to
        #       add and remove items from a top-level menu.
        self._oFirstMenuItem = None
        self._sPrefsPath = None

    def _read_index(self):
        """Read the list of rulebooks from the index.txt file.

           Return a list of (filename, title) tuples.
           """
        sIndexFile = os.path.join(self._sPrefsPath, 'index.txt')
        if not os.path.isfile(sIndexFile):
            return []
        aRulebooks = []
        for sLine in open(sIndexFile, 'rU'):
            sFilename, sSep, sTitle = sLine.partition(':')
            if not sSep:
                continue
            aRulebooks.append((sFilename.strip(), sTitle.strip()))
        return aRulebooks

    def get_menu_item(self):
        """Overrides method from base class.

           Adds the menu item on the MainWindow if the files can be found.
           """
        if not self.check_versions() or not self.check_model_type():
            return None

        # Need to set this up here, after register_with_config has been called
        self._sPrefsPath = self.get_config_item('rulebook path')
        if self._sPrefsPath is None:
            self._sPrefsPath = os.path.join(prefs_dir('Sutekh'), 'rulebook')
            self.set_config_item('rulebook path', self._sPrefsPath)

        oConfigMenuItem = gtk.MenuItem("Download Rulebook Files")
        oConfigMenuItem.connect("activate", self.config_activate)

        aMenuList = [('Data Downloads', oConfigMenuItem)]

        for oItem in self._recreate_menu_items():
            aMenuList.append(('Rulebook', oItem))

        return aMenuList

    def _recreate_menu_items(self):
        """Read the index file, update the first menu item hack
           and return the list of menu items.
           """
        aItems = []
        for sFilename, sTitle in self._read_index():
            oItem = gtk.MenuItem(sTitle)
            oItem.connect("activate", self.rulebook_activate, sFilename)
            aItems.append(oItem)
        if not aItems:
            oItem = gtk.MenuItem('No rulebooks')
            oItem.set_sensitive(False)
            aItems.append(oItem)
        self._oFirstMenuItem = aItems[0]
        return aItems

    def _update_menu(self):
        """Read index file and update which menu items are active
           accordingly.
           """
        oRulebookMenu = self._oFirstMenuItem.get_parent()
        for oChild in oRulebookMenu.get_children():
            oRulebookMenu.remove(oChild)

        for oItem in self._recreate_menu_items():
            oRulebookMenu.append(oItem)

        oRulebookMenu.show_all()

    def setup(self):
        """Prompt the user to download/setup the rulebook the first time"""
        sIndexTxt = os.path.join(self._sPrefsPath, 'index.txt')
        if not os.path.exists(sIndexTxt):
            # Looks like the first time
            oDialog = RulebookConfigDialog(self.parent, True)
            self.handle_config_response(oDialog)
            # Don't get called next time
            ensure_dir_exists(self._sPrefsPath)
            if not os.path.exists(sIndexTxt):
                fIndexTxt = open(sIndexTxt, "wb")
                fIndexTxt.close()

    def config_activate(self, _oMenuWidget):
        """Launch the configuration dialog."""
        oDialog = RulebookConfigDialog(self.parent, False)
        self.handle_config_response(oDialog)

    def rulebook_activate(self, _oMenuWidget, sFilename):
        """Show HTML file associated with the sName."""
        sPath = os.path.join(self._sPrefsPath, sFilename)
        sPath = os.path.abspath(sPath)
        sDrive, sPath = os.path.splitdrive(sPath)
        # os.path.splitdrive assumes that drives have the form "X:"
        # so do the same in our replace.
        sPath = sDrive.replace(":", "|") + sPath
        sPath = sPath.replace(os.path.sep, "/")
        if not sPath.startswith("/"):
            sPath = "/" + sPath
        sUrl = "file:///" + sPath
        webbrowser.open(sUrl)

    def _link_resource(self, sLocalUrl):
        """Return a file-like object which sLocalUrl can be read from."""
        sResource = os.path.join(self._sPrefsPath, sLocalUrl)
        if os.path.exists(sResource):
            return file(sResource, 'rb')
        else:
            raise ValueError("Unknown resource %s" % sLocalUrl)

    def handle_config_response(self, oConfigDialog):
        """Handle the response from the config dialog"""
        iResponse = oConfigDialog.run()

        if iResponse == gtk.RESPONSE_OK:
            # pylint: disable-msg=W0703
            # we want to catch all errors here
            try:
                oRawFile = StringIO.StringIO(oConfigDialog.get_data())
                oZipFile = zipfile.ZipFile(oRawFile, 'r')
                self._unpack_zipfile_with_progress_bar(oZipFile)
            except Exception:
                do_exception_complaint('Unable to successfully download or '
                                       'copy some or all of the rulebook '
                                       'files')

        self._update_menu()

        # get rid of the dialog
        oConfigDialog.destroy()

    def _unpack_zipfile_with_progress_bar(self, oZipFile):
        """Unpack the contenst of the zip file showing progress as we go."""
        oLogHandler = SutekhCountLogHandler()
        oProgressDialog = ProgressDialog()
        oProgressDialog.set_description("Importing Rulebooks")
        oLogger = Logger('Read zip file')

        oLogger.addHandler(oLogHandler)
        oLogHandler.set_dialog(oProgressDialog)
        oLogHandler.set_total(len(oZipFile.infolist()))
        oProgressDialog.show()

        try:
            self._unpack_zipfile(oZipFile, oLogger)
        finally:
            oProgressDialog.destroy()

    def _unpack_zipfile(self, oZipFile, oLogger):
        """Unpack the contents of the zip file into the preferences folder."""
        ensure_dir_exists(self._sPrefsPath)

        for oItem in oZipFile.infolist():
            sBasename = os.path.basename(oItem.filename)
            oLogger.info('Read %s' % sBasename)
            sFile = os.path.join(self._sPrefsPath, sBasename)
            oData = oZipFile.read(oItem.filename)

            oFile = open(sFile, 'wb')
            oFile.write(oData)
            oFile.close()


plugin = RulebookPlugin
